@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "video.h"
@@ -38,6 +30,7 @@ static int g_fps = 0;
 #if _WIN32 || _WIN64
 
 static DWORD g_msec = 0;
+
 #ifdef _WINDOWS
 HINSTANCE video::win_hInstance = 0;
 int video::win_iCmdShow = 0;
@@ -46,13 +39,13 @@ void video::win_load_accelerators(int idc)  { }
 #endif //_WINDOWS
 
 #else
-
 #include <sched.h>
 #include <sys/time.h>
 struct timeval g_time;
-
 #endif //_WIN32||_WIN64
 
+#define CALC_FPS_ENABLED ((WINAPI_FAMILY != WINAPI_FAMILY_APP) && (!__ANDROID__))
+
 video::video()
     // OpenGL* RGBA byte order for little-endian CPU
     : red_mask(0xff), red_shift(0), green_mask(0xff00),
@@ -78,6 +71,7 @@ bool video::init_console()
 
 void video::terminate()
 {
+#if CALC_FPS_ENABLED
     if(calc_fps) {
         double fps = g_fps;
 #if _WIN32 || _WIN64
@@ -87,7 +81,8 @@ void video::terminate()
         fps /= (end_time.tv_sec+1.0*end_time.tv_usec/1000000.0) - (g_time.tv_sec+1.0*g_time.tv_usec/1000000.0);
 #endif
         printf("%s: %.1f fps\n", title, fps);
-  	}
+    }
+#endif
     g_video = 0; running = false;
     if(g_pImg) { delete[] g_pImg; g_pImg = 0; }
 }
@@ -100,16 +95,18 @@ video::~video()
 //! Count and display FPS count in titlebar
 bool video::next_frame()
 {
+#if CALC_FPS_ENABLED
     if(calc_fps){
-	    if(!g_fps) {
+        if(!g_fps) {
 #if _WIN32 || _WIN64
             g_msec = GetTickCount();
 #else
             struct timezone tz; gettimeofday(&g_time, &tz);
 #endif
-	    }
+        }
         g_fps++;
     }
+#endif
     return running;
 }
 
