@@ -1,43 +1,41 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef UNIVERSE_H_
 #define UNIVERSE_H_
 
+#ifndef UNIVERSE_WIDTH
+#define UNIVERSE_WIDTH 1024
+#endif
+#ifndef UNIVERSE_HEIGHT
+#define UNIVERSE_HEIGHT 512
+#endif
+
 #include "../../common/gui/video.h"
 #include "tbb/partitioner.h"
 
-
 class Universe {
 public:
     enum {
-        UniverseWidth = 1024
-        ,UniverseHeight  = 512
+        UniverseWidth  = UNIVERSE_WIDTH,
+        UniverseHeight = UNIVERSE_HEIGHT
     };
 private:
     //in order to avoid performance degradation due to cache aliasing issue
@@ -46,7 +44,7 @@ private:
     //as the compiler is forced to place class members of the same clause in order of the
     //declaration this seems to be the right way of padding.
 
-    //magic constants added below are chosen experimentally.
+    //magic constants added below are chosen experimentally for 1024x512.
     enum {
         MaxWidth = UniverseWidth+1,
         MaxHeight = UniverseHeight+3
