@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 /* Container implementations in this header are based on PPL implementations
@@ -69,14 +61,11 @@ class concurrent_unordered_set : public internal::concurrent_unordered_base< con
     typedef internal::hash_compare<Key, Hasher, Key_equality> hash_compare;
     typedef internal::concurrent_unordered_base< concurrent_unordered_set_traits<Key, hash_compare, Allocator, false> > base_type;
     typedef concurrent_unordered_set_traits<Key, internal::hash_compare<Key, Hasher, Key_equality>, Allocator, false> traits_type;
-    using traits_type::my_hash_compare;
 #if __TBB_EXTRA_DEBUG
 public:
 #endif
     using traits_type::allow_multimapping;
 public:
-    using base_type::end;
-    using base_type::find;
     using base_type::insert;
 
     // Type definitions
@@ -102,28 +91,27 @@ public:
     typedef typename base_type::const_iterator const_local_iterator;
 
     // Construction/destruction/copying
-    explicit concurrent_unordered_set(size_type n_of_buckets = 8, const hasher& a_hasher = hasher(),
+    explicit concurrent_unordered_set(size_type n_of_buckets = base_type::initial_bucket_number, const hasher& a_hasher = hasher(),
         const key_equal& a_keyeq = key_equal(), const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(a_hasher, a_keyeq), a)
     {
     }
 
-    concurrent_unordered_set(const Allocator& a) : base_type(8, key_compare(), a)
+    concurrent_unordered_set(const Allocator& a) : base_type(base_type::initial_bucket_number, key_compare(), a)
     {
     }
 
     template <typename Iterator>
-    concurrent_unordered_set(Iterator first, Iterator last, size_type n_of_buckets = 8, const hasher& a_hasher = hasher(),
+    concurrent_unordered_set(Iterator first, Iterator last, size_type n_of_buckets = base_type::initial_bucket_number, const hasher& a_hasher = hasher(),
         const key_equal& a_keyeq = key_equal(), const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(a_hasher, a_keyeq), a)
     {
-        for (; first != last; ++first)
-            base_type::insert(*first);
+        insert(first, last);
     }
 
 #if __TBB_INITIALIZER_LISTS_PRESENT
     //! Constructor from initializer_list
-   concurrent_unordered_set(std::initializer_list<value_type> const& il, size_type n_of_buckets = 8, const hasher& a_hasher = hasher(),
+   concurrent_unordered_set(std::initializer_list<value_type> il, size_type n_of_buckets = base_type::initial_bucket_number, const hasher& a_hasher = hasher(),
         const key_equal& a_keyeq = key_equal(), const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(a_hasher, a_keyeq), a)
     {
@@ -131,60 +119,40 @@ public:
     }
 #endif //# __TBB_INITIALIZER_LISTS_PRESENT
 
-    concurrent_unordered_set(const concurrent_unordered_set& table) : base_type(table)
+#if __TBB_CPP11_RVALUE_REF_PRESENT && __TBB_CPP11_IMPLICIT_MOVE_MEMBERS_GENERATION_FOR_DERIVED_BROKEN
+    concurrent_unordered_set(const concurrent_unordered_set& table)
+        : base_type(table)
     {
     }
 
+    concurrent_unordered_set& operator=(const concurrent_unordered_set& table)
+    {
+        return static_cast<concurrent_unordered_set&>(base_type::operator=(table));
+    }
+
+    concurrent_unordered_set(concurrent_unordered_set&& table)
+        : base_type(std::move(table))
+    {
+    }
+
+    concurrent_unordered_set& operator=(concurrent_unordered_set&& table)
+    {
+        return static_cast<concurrent_unordered_set&>(base_type::operator=(std::move(table)));
+    }
+#endif //__TBB_CPP11_IMPLICIT_MOVE_MEMBERS_GENERATION_FOR_DERIVED_BROKEN
+
     concurrent_unordered_set(const concurrent_unordered_set& table, const Allocator& a)
         : base_type(table, a)
     {
     }
 
-    concurrent_unordered_set& operator=(const concurrent_unordered_set& table)
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    concurrent_unordered_set(concurrent_unordered_set&& table, const Allocator& a)
+        : base_type(std::move(table), a)
     {
-        base_type::operator=(table);
-        return (*this);
     }
+#endif //__TBB_CPP11_RVALUE_REF_PRESENT
 
-#if __TBB_INITIALIZER_LISTS_PRESENT
-    //! assignment operator from initializer_list
-    concurrent_unordered_set& operator=(std::initializer_list<value_type> const& il)
-    {
-        base_type::operator=(il);
-        return (*this);
-    }
-#endif //# __TBB_INITIALIZER_LISTS_PRESENT
-
-    iterator unsafe_erase(const_iterator where)
-    {
-        return base_type::unsafe_erase(where);
-    }
-
-    size_type unsafe_erase(const key_type& key)
-    {
-        return base_type::unsafe_erase(key);
-    }
-
-    iterator unsafe_erase(const_iterator first, const_iterator last)
-    {
-        return base_type::unsafe_erase(first, last);
-    }
-
-    void swap(concurrent_unordered_set& table)
-    {
-        base_type::swap(table);
-    }
-
-    // Observers
-    hasher hash_function() const
-    {
-        return my_hash_compare.my_hash_object;
-    }
-
-    key_equal key_eq() const
-    {
-        return my_hash_compare.my_key_compare_object;
-    }
 };
 
 template <typename Key, typename Hasher = tbb::tbb_hash<Key>, typename Key_equality = std::equal_to<Key>,
@@ -193,13 +161,16 @@ class concurrent_unordered_multiset :
     public internal::concurrent_unordered_base< concurrent_unordered_set_traits<Key,
     internal::hash_compare<Key, Hasher, Key_equality>, Allocator, true> >
 {
-public:
     // Base type definitions
     typedef internal::hash_compare<Key, Hasher, Key_equality> hash_compare;
     typedef concurrent_unordered_set_traits<Key, hash_compare, Allocator, true> traits_type;
     typedef internal::concurrent_unordered_base< traits_type > base_type;
+#if __TBB_EXTRA_DEBUG
+public:
+#endif
     using traits_type::allow_multimapping;
-    using traits_type::my_hash_compare;
+public:
+    using base_type::insert;
 
     // Type definitions
     typedef Key key_type;
@@ -224,32 +195,29 @@ public:
     typedef typename base_type::const_iterator const_local_iterator;
 
     // Construction/destruction/copying
-    explicit concurrent_unordered_multiset(size_type n_of_buckets = 8,
+    explicit concurrent_unordered_multiset(size_type n_of_buckets = base_type::initial_bucket_number,
         const hasher& _Hasher = hasher(), const key_equal& _Key_equality = key_equal(),
         const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(_Hasher, _Key_equality), a)
     {
     }
 
-    concurrent_unordered_multiset(const Allocator& a) : base_type(8, key_compare(), a)
+    concurrent_unordered_multiset(const Allocator& a) : base_type(base_type::initial_bucket_number, key_compare(), a)
     {
     }
 
     template <typename Iterator>
-    concurrent_unordered_multiset(Iterator first, Iterator last, size_type n_of_buckets = 8,
+    concurrent_unordered_multiset(Iterator first, Iterator last, size_type n_of_buckets = base_type::initial_bucket_number,
         const hasher& _Hasher = hasher(), const key_equal& _Key_equality = key_equal(),
         const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(_Hasher, _Key_equality), a)
     {
-        for (; first != last; ++first)
-        {
-            base_type::insert(*first);
-        }
+        insert(first, last);
     }
-    
+
 #if __TBB_INITIALIZER_LISTS_PRESENT
     //! Constructor from initializer_list
-   concurrent_unordered_multiset(std::initializer_list<value_type> const& il, size_type n_of_buckets = 8, const hasher& a_hasher = hasher(),
+   concurrent_unordered_multiset(std::initializer_list<value_type> il, size_type n_of_buckets = base_type::initial_bucket_number, const hasher& a_hasher = hasher(),
         const key_equal& a_keyeq = key_equal(), const allocator_type& a = allocator_type())
         : base_type(n_of_buckets, key_compare(a_hasher, a_keyeq), a)
     {
@@ -257,76 +225,39 @@ public:
     }
 #endif //# __TBB_INITIALIZER_LISTS_PRESENT    
 
-    concurrent_unordered_multiset(const concurrent_unordered_multiset& table) : base_type(table)
+#if __TBB_CPP11_RVALUE_REF_PRESENT && __TBB_CPP11_IMPLICIT_MOVE_MEMBERS_GENERATION_FOR_DERIVED_BROKEN
+   concurrent_unordered_multiset(const concurrent_unordered_multiset& table)
+        : base_type(table)
     {
     }
 
-    concurrent_unordered_multiset(const concurrent_unordered_multiset& table, const Allocator& a) : base_type(table, a)
+   concurrent_unordered_multiset& operator=(const concurrent_unordered_multiset& table)
+    {
+        return static_cast<concurrent_unordered_multiset&>(base_type::operator=(table));
+    }
+
+   concurrent_unordered_multiset(concurrent_unordered_multiset&& table)
+        : base_type(std::move(table))
     {
     }
 
-    concurrent_unordered_multiset& operator=(const concurrent_unordered_multiset& table)
+   concurrent_unordered_multiset& operator=(concurrent_unordered_multiset&& table)
+    {
+        return static_cast<concurrent_unordered_multiset&>(base_type::operator=(std::move(table)));
+    }
+#endif //__TBB_CPP11_IMPLICIT_MOVE_MEMBERS_GENERATION_FOR_DERIVED_BROKEN
+
+    concurrent_unordered_multiset(const concurrent_unordered_multiset& table, const Allocator& a)
+        : base_type(table, a)
     {
-        base_type::operator=(table);
-        return (*this);
     }
 
-#if __TBB_INITIALIZER_LISTS_PRESENT
-    //! assignment operator from initializer_list
-    concurrent_unordered_multiset& operator=(std::initializer_list<value_type> const& il)
+#if __TBB_CPP11_RVALUE_REF_PRESENT
+    concurrent_unordered_multiset(concurrent_unordered_multiset&& table, const Allocator& a)
+        : base_type(std::move(table), a)
     {
-        base_type::operator=(il);
-        return (*this);
-    }
-#endif //# __TBB_INITIALIZER_LISTS_PRESENT
-
-    // Modifiers
-    std::pair<iterator, bool> insert(const value_type& value)
-    {
-        return base_type::insert(value);
-    }
-
-    iterator insert(const_iterator where, const value_type& value)
-    {
-        return base_type::insert(where, value);
-    }
-
-    template<class Iterator>
-    void insert(Iterator first, Iterator last)
-    {
-        base_type::insert(first, last);
-    }
-
-    iterator unsafe_erase(const_iterator where)
-    {
-        return base_type::unsafe_erase(where);
-    }
-
-    size_type unsafe_erase(const key_type& key)
-    {
-        return base_type::unsafe_erase(key);
-    }
-
-    iterator unsafe_erase(const_iterator first, const_iterator last)
-    {
-        return base_type::unsafe_erase(first, last);
-    }
-
-    void swap(concurrent_unordered_multiset& table)
-    {
-        base_type::swap(table);
-    }
-
-    // Observers
-    hasher hash_function() const
-    {
-        return my_hash_compare.my_hash_object;
-    }
-
-    key_equal key_eq() const
-    {
-        return my_hash_compare.my_key_compare_object;
     }
+#endif //__TBB_CPP11_RVALUE_REF_PRESENT
 };
 } // namespace interface5
 
