@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef __TBB__flow_graph_node_impl_H
@@ -59,35 +51,51 @@ namespace internal {
     //  call and any handling of the result.
     template< typename Input, typename A, typename ImplType >
     class function_input_base : public receiver<Input>, tbb::internal::no_assign {
-        typedef sender<Input> predecessor_type;
         enum op_stat {WAIT=0, SUCCEEDED, FAILED};
-        enum op_type {reg_pred, rem_pred, app_body, try_fwd, tryput_bypass, app_body_bypass };
-        typedef function_input_base<Input, A, ImplType> my_class;
+        enum op_type {reg_pred, rem_pred, app_body, try_fwd, tryput_bypass, app_body_bypass
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+            , add_blt_pred, del_blt_pred,
+            blt_pred_cnt, blt_pred_cpy   // create vector copies of preds and succs
+#endif 
+        };
+        typedef function_input_base<Input, A, ImplType> class_type;
         
     public:
 
         //! The input type of this receiver
         typedef Input input_type;
-        
+        typedef sender<Input> predecessor_type;
+        typedef predecessor_cache<input_type, null_mutex > predecessor_cache_type;
+        typedef function_input_queue<input_type, A> input_queue_type;
+        typedef typename A::template rebind< input_queue_type >::other queue_allocator_type;
+
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        typedef typename predecessor_cache_type::built_predecessors_type built_predecessors_type;
+        typedef typename receiver<input_type>::predecessor_list_type predecessor_list_type;
+#endif
+
         //! Constructor for function_input_base
-        function_input_base( graph &g, size_t max_concurrency, function_input_queue<input_type,A> *q = NULL )
-            : my_root_task(g.root_task()), my_max_concurrency(max_concurrency), my_concurrency(0),
+        function_input_base( graph &g, size_t max_concurrency, input_queue_type *q = NULL)
+            : my_graph(g), my_max_concurrency(max_concurrency), my_concurrency(0),
               my_queue(q), forwarder_busy(false) {
             my_predecessors.set_owner(this);
-            my_aggregator.initialize_handler(my_handler(this));
+            my_aggregator.initialize_handler(handler_type(this));
         }
         
         //! Copy constructor
-        function_input_base( const function_input_base& src, function_input_queue<input_type,A> *q = NULL ) :
+        function_input_base( const function_input_base& src, input_queue_type *q = NULL) :
             receiver<Input>(), tbb::internal::no_assign(),
-            my_root_task( src.my_root_task), my_max_concurrency(src.my_max_concurrency),
+            my_graph(src.my_graph), my_max_concurrency(src.my_max_concurrency),
             my_concurrency(0), my_queue(q), forwarder_busy(false)
         {
             my_predecessors.set_owner(this);
-            my_aggregator.initialize_handler(my_handler(this));
+            my_aggregator.initialize_handler(handler_type(this));
         }
 
         //! Destructor
+        // The queue is allocated by the constructor for {multi}function_node.
+        // TODO: pass the graph_buffer_policy to the base so it can allocate the queue instead.
+        // This would be an interface-breaking change.
         virtual ~function_input_base() { 
             if ( my_queue ) delete my_queue;
         }
@@ -97,7 +105,7 @@ namespace internal {
            if ( my_max_concurrency == 0 ) {
                return create_body_task( t );
            } else {
-               my_operation op_data(t, tryput_bypass);
+               operation_type op_data(t, tryput_bypass);
                my_aggregator.execute(&op_data);
                if(op_data.status == SUCCEEDED ) {
                    return op_data.bypass_t;
@@ -108,7 +116,7 @@ namespace internal {
 
         //! Adds src to the list of cached predecessors.
         /* override */ bool register_predecessor( predecessor_type &src ) {
-            my_operation op_data(reg_pred);
+            operation_type op_data(reg_pred);
             op_data.r = &src;
             my_aggregator.execute(&op_data);
             return true;
@@ -116,58 +124,97 @@ namespace internal {
         
         //! Removes src from the list of cached predecessors.
         /* override */ bool remove_predecessor( predecessor_type &src ) {
-            my_operation op_data(rem_pred);
+            operation_type op_data(rem_pred);
             op_data.r = &src;
             my_aggregator.execute(&op_data);
             return true;
         }
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        //! Adds to list of predecessors added by make_edge
+        /*override*/ void internal_add_built_predecessor( predecessor_type &src) {
+            operation_type op_data(add_blt_pred);
+            op_data.r = &src;
+            my_aggregator.execute(&op_data);
+        }
+
+        //! removes from to list of predecessors (used by remove_edge)
+        /*override*/ void internal_delete_built_predecessor( predecessor_type &src) {
+            operation_type op_data(del_blt_pred);
+            op_data.r = &src;
+            my_aggregator.execute(&op_data);
+        }
+
+        /*override*/ size_t predecessor_count() {
+            operation_type op_data(blt_pred_cnt);
+            my_aggregator.execute(&op_data);
+            return op_data.cnt_val;
+        }
+
+        /*override*/ void copy_predecessors(predecessor_list_type &v) {
+            operation_type op_data(blt_pred_cpy);
+            op_data.predv = &v;
+            my_aggregator.execute(&op_data);
+        }
+
+        /*override*/built_predecessors_type &built_predecessors() {
+            return my_predecessors.built_predecessors();
+        }
+#endif  /* TBB_PREVIEW_FLOW_GRAPH_FEATURES */
+
     protected:
 
-        void reset_function_input_base() {
+        void reset_function_input_base( reset_flags f) {
             my_concurrency = 0;
             if(my_queue) {
                 my_queue->reset();
             }
-            my_predecessors.reset();
+            reset_receiver(f);
             forwarder_busy = false;
         }
 
-        task *my_root_task;
+        graph& my_graph;
         const size_t my_max_concurrency;
         size_t my_concurrency;
-        function_input_queue<input_type, A> *my_queue;
+        input_queue_type *my_queue;
         predecessor_cache<input_type, null_mutex > my_predecessors;
         
-        /*override*/void reset_receiver() {
-            my_predecessors.reset();
+        /*override*/void reset_receiver( reset_flags f) {
+            if( f & rf_clear_edges) my_predecessors.clear();
+            else
+                my_predecessors.reset();
+            __TBB_ASSERT(!(f & rf_clear_edges) || my_predecessors.empty(), "function_input_base reset failed");
         }
 
     private:
 
-        friend class apply_body_task_bypass< my_class, input_type >;
-        friend class forward_task_bypass< my_class >;
+        friend class apply_body_task_bypass< class_type, input_type >;
+        friend class forward_task_bypass< class_type >;
         
-        class my_operation : public aggregated_operation< my_operation > {
+        class operation_type : public aggregated_operation< operation_type > {
         public:
             char type;
             union {
                 input_type *elem;
                 predecessor_type *r;
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+                size_t cnt_val;
+                predecessor_list_type *predv;
+#endif  /* TBB_PREVIEW_FLOW_GRAPH_FEATURES */
             };
             tbb::task *bypass_t;
-            my_operation(const input_type& e, op_type t) :
+            operation_type(const input_type& e, op_type t) :
                 type(char(t)), elem(const_cast<input_type*>(&e)) {}
-            my_operation(op_type t) : type(char(t)), r(NULL) {}
+            operation_type(op_type t) : type(char(t)), r(NULL) {}
         };
         
         bool forwarder_busy;
-        typedef internal::aggregating_functor<my_class, my_operation> my_handler;
-        friend class internal::aggregating_functor<my_class, my_operation>;
-        aggregator< my_handler, my_operation > my_aggregator;
+        typedef internal::aggregating_functor<class_type, operation_type> handler_type;
+        friend class internal::aggregating_functor<class_type, operation_type>;
+        aggregator< handler_type, operation_type > my_aggregator;
         
-        void handle_operations(my_operation *op_list) {
-            my_operation *tmp;
+        void handle_operations(operation_type *op_list) {
+            operation_type *tmp;
             while (op_list) {
                 tmp = op_list;
                 op_list = op_list->next;
@@ -223,12 +270,31 @@ namespace internal {
                     break;
                 case tryput_bypass: internal_try_put_task(tmp);  break;
                 case try_fwd: internal_forward(tmp);  break;
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+                case add_blt_pred: {
+                         my_predecessors.internal_add_built_predecessor(*(tmp->r));
+                        __TBB_store_with_release(tmp->status, SUCCEEDED);
                     }
+                    break;
+                case del_blt_pred:
+                    my_predecessors.internal_delete_built_predecessor(*(tmp->r));
+                    __TBB_store_with_release(tmp->status, SUCCEEDED);
+                    break;
+                case blt_pred_cnt:
+                    tmp->cnt_val = my_predecessors.predecessor_count();
+                    __TBB_store_with_release(tmp->status, SUCCEEDED);
+                    break;
+                case blt_pred_cpy:
+                    my_predecessors.copy_predecessors( *(tmp->predv) );
+                    __TBB_store_with_release(tmp->status, SUCCEEDED);
+                    break;
+#endif  /* TBB_PREVIEW_FLOW_GRAPH_FEATURES */
+                }
             }
         }
         
         //! Put to the node, but return the task instead of enqueueing it
-        void internal_try_put_task(my_operation *op) {
+        void internal_try_put_task(operation_type *op) {
             __TBB_ASSERT(my_max_concurrency != 0, NULL);
             if (my_concurrency < my_max_concurrency) {
                ++my_concurrency;
@@ -245,7 +311,7 @@ namespace internal {
         }
         
         //! Tries to spawn bodies if available and if concurrency allows
-        void internal_forward(my_operation *op) {
+        void internal_forward(operation_type *op) {
             op->bypass_t = NULL;
             if (my_concurrency<my_max_concurrency || !my_max_concurrency) {
                 input_type i;
@@ -264,23 +330,13 @@ namespace internal {
             __TBB_store_with_release(op->status, FAILED);
             forwarder_busy = false;
         }
-        
-        //! Applies the body to the provided input
-        //  then decides if more work is available 
-        void apply_body( input_type &i ) {
-            task *new_task = apply_body_bypass(i);
-            if(!new_task) return;
-            if(new_task == SUCCESSFULLY_ENQUEUED) return;
-            FLOW_SPAWN(*new_task);
-            return;
-        }
-        
+
         //! Applies the body to the provided input
         //  then decides if more work is available 
         task * apply_body_bypass( input_type &i ) {
             task * new_task = static_cast<ImplType *>(this)->apply_body_impl_bypass(i);
             if ( my_max_concurrency != 0 ) {
-                my_operation op_data(app_body_bypass);  // tries to pop an item or get_item, enqueues another apply_body
+                operation_type op_data(app_body_bypass);  // tries to pop an item or get_item, enqueues another apply_body
                 my_aggregator.execute(&op_data);
                 tbb::task *ttask = op_data.bypass_t;
                 new_task = combine_tasks(new_task, ttask);
@@ -288,20 +344,27 @@ namespace internal {
             return new_task;
         }
         
-       //! allocates a task to call apply_body( input )
-       inline task * create_body_task( const input_type &input ) {
-           return new(task::allocate_additional_child_of(*my_root_task))
-               apply_body_task_bypass < my_class, input_type >(*this, input);
-       }
+        //! allocates a task to apply a body
+        inline task * create_body_task( const input_type &input ) {
+            
+            return (my_graph.is_active()) ?
+                new(task::allocate_additional_child_of(*(my_graph.root_task())))
+                    apply_body_task_bypass < class_type, input_type >(*this, input) :
+                NULL;
+        }
 
-       //! Spawns a task that calls apply_body( input )
+       //! Spawns a task that applies a body
        inline void spawn_body_task( const input_type &input ) {
-           FLOW_SPAWN(*create_body_task(input));
+           task* tp = create_body_task(input);
+           // tp == NULL => g.reset(), which shouldn't occur in concurrent context
+           if(tp) {
+               FLOW_SPAWN(*tp);
+           }
        }
         
        //! This is executed by an enqueued task, the "forwarder"
        task *forward_task() {
-           my_operation op_data(try_fwd);
+           operation_type op_data(try_fwd);
            task *rval = NULL;
            do {
                op_data.status = WAIT;
@@ -315,13 +378,17 @@ namespace internal {
        }
         
        inline task *create_forward_task() {
-           task *rval = new(task::allocate_additional_child_of(*my_root_task)) forward_task_bypass< my_class >(*this);
-           return rval;
+           return (my_graph.is_active()) ?
+               new(task::allocate_additional_child_of(*(my_graph.root_task()))) forward_task_bypass< class_type >(*this) :
+               NULL;
        }
 
        //! Spawns a task that calls forward()
        inline void spawn_forward_task() {
-           FLOW_SPAWN(*create_forward_task());
+           task* tp = create_forward_task();
+           if(tp) {
+               FLOW_SPAWN(*tp);
+           }
        }
     };  // function_input_base
 
@@ -332,93 +399,156 @@ namespace internal {
     public:
         typedef Input input_type;
         typedef Output output_type;
+        typedef function_body<input_type, output_type> function_body_type;
         typedef function_input<Input,Output,A> my_class;
         typedef function_input_base<Input, A, my_class> base_type;
         typedef function_input_queue<input_type, A> input_queue_type;
 
-
         // constructor
         template<typename Body>
-        function_input( graph &g, size_t max_concurrency, Body& body, function_input_queue<input_type,A> *q = NULL ) :
+        function_input( graph &g, size_t max_concurrency, Body& body, input_queue_type *q = NULL ) :
             base_type(g, max_concurrency, q),
-            my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ) {
+            my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ),
+            my_init_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ) {
         }
 
         //! Copy constructor
         function_input( const function_input& src, input_queue_type *q = NULL ) : 
                 base_type(src, q),
-                my_body( src.my_body->clone() ) {
+                my_body( src.my_init_body->clone() ),
+                my_init_body(src.my_init_body->clone() ) {
         }
 
         ~function_input() {
             delete my_body;
+            delete my_init_body;
         }
 
         template< typename Body >
         Body copy_function_object() {
-            internal::function_body<input_type, output_type> &body_ref = *this->my_body;
+            function_body_type &body_ref = *this->my_body;
             return dynamic_cast< internal::function_body_leaf<input_type, output_type, Body> & >(body_ref).get_body(); 
         } 
 
         task * apply_body_impl_bypass( const input_type &i) {
+#if TBB_PREVIEW_FLOW_GRAPH_TRACE
+            // There is an extra copied needed to capture the
+            // body execution without the try_put
+            tbb::internal::fgt_begin_body( my_body );
+            output_type v = (*my_body)(i);
+            tbb::internal::fgt_end_body( my_body );
+            task * new_task = successors().try_put_task( v );
+#else       
             task * new_task = successors().try_put_task( (*my_body)(i) );
+#endif
             return new_task;
         }
 
     protected:
 
-        void reset_function_input() { 
-            base_type::reset_function_input_base();
+        void reset_function_input(reset_flags f) {
+            base_type::reset_function_input_base(f);
+            if(f & rf_reset_bodies) {
+                function_body_type *tmp = my_init_body->clone();
+                delete my_body;
+                my_body = tmp;
+            }
         }
 
-        function_body<input_type, output_type> *my_body;
+        function_body_type *my_body;
+        function_body_type *my_init_body;
         virtual broadcast_cache<output_type > &successors() = 0;
 
+    };  // function_input
+
+
+    // helper templates to clear the successor edges of the output ports of an multifunction_node
+    template<int N> struct clear_element {
+        template<typename P> static void clear_this(P &p) {
+            (void)tbb::flow::get<N-1>(p).successors().clear();
+            clear_element<N-1>::clear_this(p);
+        }
+        template<typename P> static bool this_empty(P &p) {
+            if(tbb::flow::get<N-1>(p).successors().empty()) 
+                return clear_element<N-1>::this_empty(p);
+            return false;
+        }
     };
 
+    template<> struct clear_element<1> {
+        template<typename P> static void clear_this(P &p) {
+            (void)tbb::flow::get<0>(p).successors().clear();
+        }
+        template<typename P> static bool this_empty(P &p) {
+            return tbb::flow::get<0>(p).successors().empty();
+        }
+    };
+
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    // helper templates to extract the output ports of an multifunction_node from graph
+    template<int N> struct extract_element {
+        template<typename P> static void extract_this(P &p) {
+            (void)tbb::flow::get<N-1>(p).successors().built_successors().sender_extract(tbb::flow::get<N-1>(p));
+            extract_element<N-1>::extract_this(p);
+        }
+    };
+
+    template<> struct extract_element<1> {
+        template<typename P> static void extract_this(P &p) {
+            (void)tbb::flow::get<0>(p).successors().built_successors().sender_extract(tbb::flow::get<0>(p));
+        }
+    };
+#endif
+
     //! Implements methods for a function node that takes a type Input as input
     //  and has a tuple of output ports specified.  
     template< typename Input, typename OutputPortSet, typename A>
     class multifunction_input : public function_input_base<Input, A, multifunction_input<Input,OutputPortSet,A> > {
     public:
+        static const int N = tbb::flow::tuple_size<OutputPortSet>::value;
         typedef Input input_type;
         typedef OutputPortSet output_ports_type;
+        typedef multifunction_body<input_type, output_ports_type> multifunction_body_type;
         typedef multifunction_input<Input,OutputPortSet,A> my_class;
         typedef function_input_base<Input, A, my_class> base_type;
         typedef function_input_queue<input_type, A> input_queue_type;
 
-
         // constructor
         template<typename Body>
-        multifunction_input( 
-                graph &g, 
-                size_t max_concurrency, 
+        multifunction_input(
+                graph &g,
+                size_t max_concurrency,
                 Body& body,
-                function_input_queue<input_type,A> *q = NULL ) :
+                input_queue_type *q = NULL ) :
             base_type(g, max_concurrency, q),
-            my_body( new internal::multifunction_body_leaf<input_type, output_ports_type, Body>(body) ) {
+            my_body( new internal::multifunction_body_leaf<input_type, output_ports_type, Body>(body) ),
+            my_init_body( new internal::multifunction_body_leaf<input_type, output_ports_type, Body>(body) ) {
         }
 
         //! Copy constructor
         multifunction_input( const multifunction_input& src, input_queue_type *q = NULL ) : 
                 base_type(src, q),
-                my_body( src.my_body->clone() ) {
+                my_body( src.my_init_body->clone() ),
+                my_init_body(src.my_init_body->clone() ) {
         }
 
         ~multifunction_input() {
             delete my_body;
+            delete my_init_body;
         }
 
         template< typename Body >
         Body copy_function_object() {
-            internal::multifunction_body<input_type, output_ports_type> &body_ref = *this->my_body;
+            multifunction_body_type &body_ref = *this->my_body;
             return dynamic_cast< internal::multifunction_body_leaf<input_type, output_ports_type, Body> & >(body_ref).get_body(); 
         } 
 
         // for multifunction nodes we do not have a single successor as such.  So we just tell
         // the task we were successful.
         task * apply_body_impl_bypass( const input_type &i) {
+            tbb::internal::fgt_begin_body( my_body );
             (*my_body)(i, my_output_ports);
+            tbb::internal::fgt_end_body( my_body );
             task * new_task = SUCCESSFULLY_ENQUEUED;
             return new_task;
         }
@@ -426,15 +556,28 @@ namespace internal {
         output_ports_type &output_ports(){ return my_output_ports; }
 
     protected:
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        /*override*/void extract() {
+            extract_element<N>::extract_this(my_output_ports);
+        }
+#endif
 
-        void reset() {
-            base_type::reset_function_input_base();
+        /*override*/void reset(reset_flags f) { 
+            base_type::reset_function_input_base(f);
+            if(f & rf_clear_edges)clear_element<N>::clear_this(my_output_ports);
+            if(f & rf_reset_bodies) {
+                multifunction_body_type *tmp = my_init_body->clone();
+                delete my_body;
+                my_body = tmp;
+            }
+            __TBB_ASSERT(!(f & rf_clear_edges) || clear_element<N>::this_empty(my_output_ports), "multifunction_node reset failed");
         }
 
-        multifunction_body<input_type, output_ports_type> *my_body;
+        multifunction_body_type *my_body;
+        multifunction_body_type *my_init_body;
         output_ports_type my_output_ports;
 
-    };
+    };  // multifunction_input
 
     // template to refer to an output port of a multifunction_node
     template<size_t N, typename MOP>
@@ -470,59 +613,93 @@ namespace internal {
             
         //! The output type of this receiver
         typedef Output output_type;
+        typedef function_body<input_type, output_type> function_body_type;
         
         template< typename Body >
         continue_input( graph &g, Body& body )
-            : my_root_task(g.root_task()), 
-             my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ) { }
+            : my_graph_ptr(&g), 
+             my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ),
+             my_init_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ) { }
         
         template< typename Body >
         continue_input( graph &g, int number_of_predecessors, Body& body )
-            : continue_receiver( number_of_predecessors ), my_root_task(g.root_task()), 
-             my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ) { }
+            : continue_receiver( number_of_predecessors ), my_graph_ptr(&g), 
+             my_body( new internal::function_body_leaf< input_type, output_type, Body>(body) ),
+             my_init_body( new internal::function_body_leaf< input_type, output_type, Body>(body) )
+        { }
 
         continue_input( const continue_input& src ) : continue_receiver(src), 
-            my_root_task(src.my_root_task), my_body( src.my_body->clone() ) {}
+            my_graph_ptr(src.my_graph_ptr),
+            my_body( src.my_init_body->clone() ),
+            my_init_body( src.my_init_body->clone() ) {}
 
         ~continue_input() {
             delete my_body;
+            delete my_init_body;
         }
 
         template< typename Body >
         Body copy_function_object() {
-            internal::function_body<input_type, output_type> &body_ref = *my_body;
+            function_body_type &body_ref = *my_body;
             return dynamic_cast< internal::function_body_leaf<input_type, output_type, Body> & >(body_ref).get_body(); 
         } 
 
+        /*override*/void reset_receiver( reset_flags f) {
+            continue_receiver::reset_receiver(f);
+            if(f & rf_reset_bodies) {
+                function_body_type *tmp = my_init_body->clone();
+                delete my_body;
+                my_body = tmp;
+            }
+        }
+
     protected:
         
-        task *my_root_task;
-        function_body<input_type, output_type> *my_body;
+        graph* my_graph_ptr;
+        function_body_type *my_body;
+        function_body_type *my_init_body;
         
         virtual broadcast_cache<output_type > &successors() = 0; 
         
         friend class apply_body_task_bypass< continue_input< Output >, continue_msg >;
         
         //! Applies the body to the provided input
-        /* override */ task *apply_body_bypass( input_type ) {
+        task *apply_body_bypass( input_type ) {
+#if TBB_PREVIEW_FLOW_GRAPH_TRACE
+            // There is an extra copied needed to capture the
+            // body execution without the try_put
+            tbb::internal::fgt_begin_body( my_body );
+            output_type v = (*my_body)( continue_msg() );
+            tbb::internal::fgt_end_body( my_body );
+            return successors().try_put_task( v );
+#else   
             return successors().try_put_task( (*my_body)( continue_msg() ) );
+#endif
         }
         
         //! Spawns a task that applies the body
         /* override */ task *execute( ) {
-            task *res = new ( task::allocate_additional_child_of( *my_root_task ) ) 
-                apply_body_task_bypass< continue_input< Output >, continue_msg >( *this, continue_msg() ); 
-            return res;
+            return (my_graph_ptr->is_active()) ?
+                new ( task::allocate_additional_child_of( *(my_graph_ptr->root_task()) ) ) 
+                    apply_body_task_bypass< continue_input< Output >, continue_msg >( *this, continue_msg() ) :
+                NULL;
         }
 
-    };
-        
+    };  // continue_input
+
     //! Implements methods for both executable and function nodes that puts Output to its successors
     template< typename Output >
     class function_output : public sender<Output> {
     public:
         
+        template<int N> friend struct clear_element;
         typedef Output output_type;
+        typedef receiver<output_type> successor_type;
+        typedef broadcast_cache<output_type> broadcast_cache_type;
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        typedef typename sender<output_type>::built_successors_type built_successors_type;
+        typedef typename sender<output_type>::successor_list_type successor_list_type;
+#endif
         
         function_output() { my_successors.set_owner(this); }
         function_output(const function_output & /*other*/) : sender<output_type>() {
@@ -541,20 +718,42 @@ namespace internal {
             return true;
         }
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        built_successors_type &built_successors() { return successors().built_successors(); }
+
+
+        /*override*/ void internal_add_built_successor( receiver<output_type> &r) {
+            successors().internal_add_built_successor( r );
+        }
+
+        /*override*/ void internal_delete_built_successor( receiver<output_type> &r) {
+            successors().internal_delete_built_successor( r );
+        }
+
+        /*override*/ size_t successor_count() {
+            return successors().successor_count();
+        }
+
+        /*override*/ void  copy_successors( successor_list_type &v) {
+            successors().copy_successors(v);
+        }
+#endif  /* TBB_PREVIEW_FLOW_GRAPH_FEATURES */
+
         // for multifunction_node.  The function_body that implements
         // the node will have an input and an output tuple of ports.  To put
         // an item to a successor, the body should
         //
         //    get<I>(output_ports).try_put(output_value);
         //
+        // if task pointer is returned will always spawn and return true, else
         // return value will be bool returned from successors.try_put.
         task *try_put_task(const output_type &i) { return my_successors.try_put_task(i); }
           
+        broadcast_cache_type &successors() { return my_successors; } 
     protected:
-        broadcast_cache<output_type> my_successors;
-        broadcast_cache<output_type > &successors() { return my_successors; } 
+        broadcast_cache_type my_successors;
         
-    };
+    };  // function_output
 
     template< typename Output >
     class multifunction_output : public function_output<Output> {
@@ -572,7 +771,24 @@ namespace internal {
             if(res != SUCCESSFULLY_ENQUEUED) FLOW_SPAWN(*res);
             return true;
         }
-    };
+    };  // multifunction_output
+
+//composite_node
+#if TBB_PREVIEW_FLOW_GRAPH_TRACE && __TBB_FLOW_GRAPH_CPP11_FEATURES
+    template<typename CompositeType>
+    void add_nodes_impl(CompositeType*, bool) {}
+
+    template< typename CompositeType, typename NodeType1, typename... NodeTypes >
+    void add_nodes_impl(CompositeType *c_node, bool visible, const NodeType1& n1, const NodeTypes&... n) {
+        void *addr = const_cast<NodeType1 *>(&n1);
+
+        if(visible)
+            tbb::internal::itt_relation_add( tbb::internal::ITT_DOMAIN_FLOW, c_node, tbb::internal::FLOW_NODE, tbb::internal::__itt_relation_is_parent_of, addr, tbb::internal::FLOW_NODE );
+        else
+            tbb::internal::itt_relation_add( tbb::internal::ITT_DOMAIN_FLOW, addr, tbb::internal::FLOW_NODE, tbb::internal::__itt_relation_is_child_of, c_node, tbb::internal::FLOW_NODE );
+        add_nodes_impl(c_node, visible, n...);
+    }
+#endif
 
 }  // internal
 
