@@ -94,7 +94,7 @@ static inline int __TBB_XBOX360_DetectNumberOfWorkers()
             ((__TBB_XBOX360_HARDWARE_THREAD_MASK >> 2) & 1) +
             ((__TBB_XBOX360_HARDWARE_THREAD_MASK >> 3) & 1) +
             ((__TBB_XBOX360_HARDWARE_THREAD_MASK >> 4) & 1) +
-            ((__TBB_XBOX360_HARDWARE_THREAD_MASK >> 5) & 1) + 1;  // +1 accomodates for the master thread
+            ((__TBB_XBOX360_HARDWARE_THREAD_MASK >> 5) & 1) + 1;  // +1 accommodates for the master thread
 }
 
 static inline int __TBB_XBOX360_GetHardwareThreadIndex(int workerThreadIndex)
