@@ -1,34 +1,28 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef __TBB_parallel_do_H
 #define __TBB_parallel_do_H
 
+#include "internal/_range_iterator.h"
+#include "internal/_template_helpers.h"
 #include "task.h"
 #include "aligned_space.h"
 #include <iterator>
@@ -39,26 +33,6 @@ namespace tbb {
 namespace internal {
     template<typename Body, typename Item> class parallel_do_feeder_impl;
     template<typename Body> class do_group_task;
-
-    //! Strips its template type argument from 'cv' and '&' qualifiers
-    template<typename T>
-    struct strip { typedef T type; };
-    template<typename T>
-    struct strip<T&> { typedef T type; };
-    template<typename T>
-    struct strip<const T&> { typedef T type; };
-    template<typename T>
-    struct strip<volatile T&> { typedef T type; };
-    template<typename T>
-    struct strip<const volatile T&> { typedef T type; };
-    // Most of the compilers remove cv-qualifiers from non-reference function argument types. 
-    // But unfortunately there are those that don't.
-    template<typename T>
-    struct strip<const T> { typedef T type; };
-    template<typename T>
-    struct strip<volatile T> { typedef T type; };
-    template<typename T>
-    struct strip<const volatile T> { typedef T type; };
 } // namespace internal
 //! @endcond
 
@@ -73,6 +47,7 @@ class parallel_do_feeder: internal::no_copy
     template<typename Body_, typename Item_> friend class internal::parallel_do_feeder_impl;
 public:
     //! Add a work item to a running parallel_do.
+    // TODO: add an overload for r-value reference
     void add( const Item& item ) {internal_add(item);}
 };
 
@@ -113,13 +88,14 @@ namespace internal {
         Item my_value;
         feeder_type& my_feeder;
 
-        do_iteration_task( const Item& value, feeder_type& feeder ) : 
+        do_iteration_task( const Item& value, feeder_type& feeder ) :
             my_value(value), my_feeder(feeder)
         {}
 
-        /*override*/ 
+        /*override*/
         task* execute()
         {
+            // TODO: use move semantics for my_value
             parallel_do_operator_selector<Body, Item>::call(*my_feeder.my_body, my_value, my_feeder);
             return NULL;
         }
@@ -135,20 +111,20 @@ namespace internal {
         Iterator my_iter;
         feeder_type& my_feeder;
 
-        do_iteration_task_iter( const Iterator& iter, feeder_type& feeder ) : 
+        do_iteration_task_iter( const Iterator& iter, feeder_type& feeder ) :
             my_iter(iter), my_feeder(feeder)
         {}
 
-        /*override*/ 
+        /*override*/
         task* execute()
         {
             parallel_do_operator_selector<Body, Item>::call(*my_feeder.my_body, *my_iter, my_feeder);
             return NULL;
         }
 
-        template<typename Iterator_, typename Body_, typename Item_> friend class do_group_task_forward;    
-        template<typename Body_, typename Item_> friend class do_group_task_input;    
-        template<typename Iterator_, typename Body_, typename Item_> friend class do_task_iter;    
+        template<typename Iterator_, typename Body_, typename Item_> friend class do_group_task_forward;
+        template<typename Body_, typename Item_> friend class do_group_task_input;
+        template<typename Iterator_, typename Body_, typename Item_> friend class do_task_iter;
     }; // class do_iteration_task_iter
 
     //! For internal use only.
@@ -157,7 +133,7 @@ namespace internal {
     template<class Body, typename Item>
     class parallel_do_feeder_impl : public parallel_do_feeder<Item>
     {
-        /*override*/ 
+        /*override*/
         void internal_add( const Item& item )
         {
             typedef do_iteration_task<Body, Item> iteration_type;
@@ -194,19 +170,19 @@ namespace internal {
     //! For internal use only
     /** Unpacks a block of iterations.
         @ingroup algorithms */
-    
+
     template<typename Iterator, typename Body, typename Item>
     class do_group_task_forward: public task
     {
-        static const size_t max_arg_size = 4;         
+        static const size_t max_arg_size = 4;
 
         typedef parallel_do_feeder_impl<Body, Item> feeder_type;
 
         feeder_type& my_feeder;
         Iterator my_first;
         size_t my_size;
-        
-        do_group_task_forward( Iterator first, size_t size, feeder_type& feeder ) 
+
+        do_group_task_forward( Iterator first, size_t size, feeder_type& feeder )
             : my_feeder(feeder), my_first(first), my_size(size)
         {}
 
@@ -215,8 +191,8 @@ namespace internal {
             typedef do_iteration_task_iter<Iterator, Body, Item> iteration_type;
             __TBB_ASSERT( my_size>0, NULL );
             task_list list;
-            task* t; 
-            size_t k=0; 
+            task* t;
+            size_t k=0;
             for(;;) {
                 t = new( allocate_child() ) iteration_type( my_first, my_feeder );
                 ++my_first;
@@ -235,15 +211,15 @@ namespace internal {
     template<typename Body, typename Item>
     class do_group_task_input: public task
     {
-        static const size_t max_arg_size = 4;         
-        
+        static const size_t max_arg_size = 4;
+
         typedef parallel_do_feeder_impl<Body, Item> feeder_type;
 
         feeder_type& my_feeder;
         size_t my_size;
         aligned_space<Item, max_arg_size> my_arg;
 
-        do_group_task_input( feeder_type& feeder ) 
+        do_group_task_input( feeder_type& feeder )
             : my_feeder(feeder), my_size(0)
         {}
 
@@ -252,8 +228,8 @@ namespace internal {
             typedef do_iteration_task_iter<Item*, Body, Item> iteration_type;
             __TBB_ASSERT( my_size>0, NULL );
             task_list list;
-            task* t; 
-            size_t k=0; 
+            task* t;
+            size_t k=0;
             for(;;) {
                 t = new( allocate_child() ) iteration_type( my_arg.begin() + k, my_feeder );
                 if( ++k==my_size ) break;
@@ -272,7 +248,7 @@ namespace internal {
 
         template<typename Iterator_, typename Body_, typename Item_> friend class do_task_iter;
     }; // class do_group_task_input
-    
+
     //! For internal use only.
     /** Gets block of iterations and packages them into a do_group_task.
         @ingroup algorithms */
@@ -282,7 +258,7 @@ namespace internal {
         typedef parallel_do_feeder_impl<Body, Item> feeder_type;
 
     public:
-        do_task_iter( Iterator first, Iterator last , feeder_type& feeder ) : 
+        do_task_iter( Iterator first, Iterator last , feeder_type& feeder ) :
             my_first(first), my_last(last), my_feeder(feeder)
         {}
 
@@ -293,13 +269,13 @@ namespace internal {
 
         /* Do not merge run(xxx) and run_xxx() methods. They are separated in order
             to make sure that compilers will eliminate unused argument of type xxx
-            (that is will not put it on stack). The sole purpose of this argument 
+            (that is will not put it on stack). The sole purpose of this argument
             is overload resolution.
-            
-            An alternative could be using template functions, but explicit specialization 
-            of member function templates is not supported for non specialized class 
-            templates. Besides template functions would always fall back to the least 
-            efficient variant (the one for input iterators) in case of iterators having 
+
+            An alternative could be using template functions, but explicit specialization
+            of member function templates is not supported for non specialized class
+            templates. Besides template functions would always fall back to the least
+            efficient variant (the one for input iterators) in case of iterators having
             custom tags derived from basic ones. */
         /*override*/ task* execute()
         {
@@ -310,13 +286,14 @@ namespace internal {
         /** This is the most restricted variant that operates on input iterators or
             iterators with unknown tags (tags not derived from the standard ones). **/
         inline task* run( void* ) { return run_for_input_iterator(); }
-        
+
         task* run_for_input_iterator() {
             typedef do_group_task_input<Body, Item> block_type;
 
             block_type& t = *new( allocate_additional_child_of(*my_feeder.my_barrier) ) block_type(my_feeder);
-            size_t k=0; 
+            size_t k=0;
             while( !(my_first == my_last) ) {
+                // TODO: move *my_first
                 new (t.my_arg.begin() + k) Item(*my_first);
                 ++my_first;
                 if( ++k==block_type::max_arg_size ) {
@@ -340,7 +317,7 @@ namespace internal {
             typedef do_group_task_forward<Iterator, Body, Item> block_type;
 
             Iterator first = my_first;
-            size_t k=0; 
+            size_t k=0;
             while( !(my_first==my_last) ) {
                 ++my_first;
                 if( ++k==block_type::max_arg_size ) {
@@ -351,14 +328,14 @@ namespace internal {
             }
             return k==0 ? NULL : new( allocate_additional_child_of(*my_feeder.my_barrier) ) block_type(first, k, my_feeder);
         }
-        
+
         inline task* run( std::random_access_iterator_tag* ) { return run_for_random_access_iterator(); }
 
         task* run_for_random_access_iterator() {
             typedef do_group_task_forward<Iterator, Body, Item> block_type;
             typedef do_iteration_task_iter<Iterator, Body, Item> iteration_type;
-            
-            size_t k = static_cast<size_t>(my_last-my_first); 
+
+            size_t k = static_cast<size_t>(my_last-my_first);
             if( k > block_type::max_arg_size ) {
                 Iterator middle = my_first + k/2;
 
@@ -372,8 +349,8 @@ namespace internal {
                 return this;
             }else if( k != 0 ) {
                 task_list list;
-                task* t; 
-                size_t k1=0; 
+                task* t;
+                size_t k1=0;
                 for(;;) {
                     t = new( allocate_child() ) iteration_type(my_first, my_feeder);
                     ++my_first;
@@ -391,7 +368,7 @@ namespace internal {
     //! For internal use only.
     /** Implements parallel iteration over a range.
         @ingroup algorithms */
-    template<typename Iterator, typename Body, typename Item> 
+    template<typename Iterator, typename Body, typename Item>
     void run_parallel_do( Iterator first, Iterator last, const Body& body
 #if __TBB_TASK_GROUP_CONTEXT
         , task_group_context& context
@@ -415,27 +392,27 @@ namespace internal {
     //! For internal use only.
     /** Detects types of Body's operator function arguments.
         @ingroup algorithms **/
-    template<typename Iterator, typename Body, typename Item> 
+    template<typename Iterator, typename Body, typename Item>
     void select_parallel_do( Iterator first, Iterator last, const Body& body, void (Body::*)(Item) const
 #if __TBB_TASK_GROUP_CONTEXT
-        , task_group_context& context 
-#endif // __TBB_TASK_GROUP_CONTEXT 
+        , task_group_context& context
+#endif // __TBB_TASK_GROUP_CONTEXT
         )
     {
         run_parallel_do<Iterator, Body, typename strip<Item>::type>( first, last, body
 #if __TBB_TASK_GROUP_CONTEXT
             , context
-#endif // __TBB_TASK_GROUP_CONTEXT 
+#endif // __TBB_TASK_GROUP_CONTEXT
             );
     }
 
     //! For internal use only.
     /** Detects types of Body's operator function arguments.
         @ingroup algorithms **/
-    template<typename Iterator, typename Body, typename Item, typename _Item> 
+    template<typename Iterator, typename Body, typename Item, typename _Item>
     void select_parallel_do( Iterator first, Iterator last, const Body& body, void (Body::*)(Item, parallel_do_feeder<_Item>&) const
 #if __TBB_TASK_GROUP_CONTEXT
-        , task_group_context& context 
+        , task_group_context& context
 #endif // __TBB_TASK_GROUP_CONTEXT
         )
     {
@@ -452,20 +429,20 @@ namespace internal {
 
 /** \page parallel_do_body_req Requirements on parallel_do body
     Class \c Body implementing the concept of parallel_do body must define:
-    - \code 
-        B::operator()( 
+    - \code
+        B::operator()(
                 cv_item_type item,
                 parallel_do_feeder<item_type>& feeder
         ) const
-        
+
         OR
 
         B::operator()( cv_item_type& item ) const
-      \endcode                                                      Process item. 
-                                                                    May be invoked concurrently  for the same \c this but different \c item.
-                                                        
-    - \code item_type( const item_type& ) \endcode 
-                                                                    Copy a work item.
+      \endcode                                               Process item.
+                                                             May be invoked concurrently  for the same \c this but different \c item.
+
+    - \code item_type( const item_type& ) \endcode
+                                                             Copy a work item.
     - \code ~item_type() \endcode                            Destroy a work item
 **/
 
@@ -474,7 +451,7 @@ namespace internal {
 //@{
 //! Parallel iteration over a range, with optional addition of more work.
 /** @ingroup algorithms */
-template<typename Iterator, typename Body> 
+template<typename Iterator, typename Body>
 void parallel_do( Iterator first, Iterator last, const Body& body )
 {
     if ( first == last )
@@ -489,20 +466,41 @@ void parallel_do( Iterator first, Iterator last, const Body& body )
         );
 }
 
+template<typename Range, typename Body>
+void parallel_do(Range& rng, const Body& body) {
+    parallel_do(tbb::internal::first(rng), tbb::internal::last(rng), body);
+}
+
+template<typename Range, typename Body>
+void parallel_do(const Range& rng, const Body& body) {
+    parallel_do(tbb::internal::first(rng), tbb::internal::last(rng), body);
+}
+
 #if __TBB_TASK_GROUP_CONTEXT
 //! Parallel iteration over a range, with optional addition of more work and user-supplied context
 /** @ingroup algorithms */
-template<typename Iterator, typename Body> 
+template<typename Iterator, typename Body>
 void parallel_do( Iterator first, Iterator last, const Body& body, task_group_context& context  )
 {
     if ( first == last )
         return;
     internal::select_parallel_do( first, last, body, &Body::operator(), context );
 }
+
+template<typename Range, typename Body>
+void parallel_do(Range& rng, const Body& body, task_group_context& context) {
+    parallel_do(tbb::internal::first(rng), tbb::internal::last(rng), body, context);
+}
+
+template<typename Range, typename Body>
+void parallel_do(const Range& rng, const Body& body, task_group_context& context) {
+    parallel_do(tbb::internal::first(rng), tbb::internal::last(rng), body, context);
+}
+
 #endif // __TBB_TASK_GROUP_CONTEXT
 
 //@}
 
-} // namespace 
+} // namespace
 
 #endif /* __TBB_parallel_do_H */
