@@ -1,32 +1,25 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "tbb/tbb_stddef.h"
+#include "tbb/global_control.h" // global_control::active_value
 
 #include "market.h"
 #include "tbb_main.h"
@@ -40,28 +33,31 @@ namespace internal {
 void market::insert_arena_into_list ( arena& a ) {
 #if __TBB_TASK_PRIORITY
     arena_list_type &arenas = my_priority_levels[a.my_top_priority].arenas;
-    arena_list_type::iterator &next = my_priority_levels[a.my_top_priority].next_arena;
+    arena *&next = my_priority_levels[a.my_top_priority].next_arena;
 #else /* !__TBB_TASK_PRIORITY */
     arena_list_type &arenas = my_arenas;
-    arena_list_type::iterator &next = my_next_arena;
+    arena *&next = my_next_arena;
 #endif /* !__TBB_TASK_PRIORITY */
     arenas.push_front( a );
     if ( arenas.size() == 1 )
-        next = arenas.begin();
+        next = &*arenas.begin();
 }
 
 void market::remove_arena_from_list ( arena& a ) {
 #if __TBB_TASK_PRIORITY
     arena_list_type &arenas = my_priority_levels[a.my_top_priority].arenas;
-    arena_list_type::iterator &next = my_priority_levels[a.my_top_priority].next_arena;
+    arena *&next = my_priority_levels[a.my_top_priority].next_arena;
 #else /* !__TBB_TASK_PRIORITY */
     arena_list_type &arenas = my_arenas;
-    arena_list_type::iterator &next = my_next_arena;
+    arena *&next = my_next_arena;
 #endif /* !__TBB_TASK_PRIORITY */
-    __TBB_ASSERT( next != arenas.end(), NULL );
-    if ( &*next == &a )
-        if ( ++next == arenas.end() && arenas.size() > 1 )
-            next = arenas.begin();
+    arena_list_type::iterator it = next;
+    __TBB_ASSERT( it != arenas.end(), NULL );
+    if ( next == &a ) {
+        if ( ++it == arenas.end() && arenas.size() > 1 )
+            it = arenas.begin();
+        next = &*it;
+    }
     arenas.remove( a );
 }
 
@@ -69,21 +65,19 @@ void market::remove_arena_from_list ( arena& a ) {
 // market
 //------------------------------------------------------------------------
 
-market::market ( unsigned max_num_workers, size_t stack_size )
-    : my_ref_count(1)
-    , my_stack_size(stack_size)
-    , my_max_num_workers(max_num_workers)
+market::market ( unsigned workers_soft_limit, unsigned workers_hard_limit, size_t stack_size )
+    : my_num_workers_hard_limit(workers_hard_limit)
+    , my_num_workers_soft_limit(workers_soft_limit)
 #if __TBB_TASK_PRIORITY
     , my_global_top_priority(normalized_normal_priority)
     , my_global_bottom_priority(normalized_normal_priority)
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    , my_lowest_populated_level(normalized_normal_priority)
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
 #endif /* __TBB_TASK_PRIORITY */
+    , my_ref_count(1)
+    , my_stack_size(stack_size)
 {
 #if __TBB_TASK_PRIORITY
     __TBB_ASSERT( my_global_reload_epoch == 0, NULL );
-    my_priority_levels[normalized_normal_priority].workers_available = max_num_workers;
+    my_priority_levels[normalized_normal_priority].workers_available = my_num_workers_soft_limit;
 #endif /* __TBB_TASK_PRIORITY */
 
     // Once created RML server will start initializing workers that will need 
@@ -92,32 +86,69 @@ market::market ( unsigned max_num_workers, size_t stack_size )
     __TBB_ASSERT( my_server, "Failed to create RML server" );
 }
 
+static unsigned calc_workers_soft_limit(unsigned workers_soft_limit, unsigned workers_hard_limit) {
+    if( int soft_limit = market::app_parallelism_limit() )
+        workers_soft_limit = soft_limit-1;
+    else // if user set no limits (yet), use market's parameter
+        workers_soft_limit = max( governor::default_num_threads() - 1, workers_soft_limit );
+    // at least 1 worker is required to support starvation resistant tasks
+    if( workers_soft_limit == 0 )
+        workers_soft_limit = 1;
+    else if( workers_soft_limit >= workers_hard_limit )
+        workers_soft_limit = workers_hard_limit-1;
+    return workers_soft_limit;
+}
 
-market& market::global_market ( unsigned max_num_workers, size_t stack_size ) {
+market& market::global_market ( bool is_public, unsigned workers_soft_limit, size_t stack_size,
+                                bool default_concurrency_requested ) {
     global_market_mutex_type::scoped_lock lock( theMarketMutex );
     market *m = theMarket;
-    if ( m ) {
+    if( m ) {
         ++m->my_ref_count;
-        if ( m->my_stack_size < stack_size )
-            runtime_warning( "Newer master request for larger stack cannot be satisfied\n" );
+        const unsigned old_public_count = is_public? m->my_public_ref_count++ : /*any non-zero value*/1;
+        lock.release();
+        if( old_public_count==0 ) {
+            workers_soft_limit = calc_workers_soft_limit(workers_soft_limit, m->my_num_workers_hard_limit);
+            set_active_num_workers( workers_soft_limit );
+        }
+        if( m->my_num_workers_soft_limit < workers_soft_limit && !default_concurrency_requested )
+            runtime_warning( "Max number of workers has been already set to %u. "
+                             "The request for %u workers is ignored.\n",
+                              m->my_num_workers_soft_limit, workers_soft_limit );
+        if( m->my_stack_size < stack_size )
+            runtime_warning( "Thread stack size has been already set to %u. "
+                             "The request for larger stack (%u) cannot be satisfied.\n",
+                              m->my_stack_size, stack_size );
     }
     else {
-        max_num_workers = max( governor::default_num_threads() - 1, max_num_workers );
-        // at least 1 worker is required to support starvation resistant tasks
-        if( max_num_workers==0 ) max_num_workers = 1;
+        // TODO: A lot is done under theMarketMutex locked. Can anything be moved out?
+        if( stack_size == 0 )
+            stack_size = global_control::active_value(global_control::thread_stack_size);
+        // Expecting that 4P is suitable for most applications.
+        // Limit to 2P for large thread number.
+        // TODO: ask RML for max concurrency and possibly correct hard_limit
+        const unsigned factor = governor::default_num_threads()<=128? 4 : 2;
+        unsigned workers_hard_limit = max(factor*governor::default_num_threads(), app_parallelism_limit());
+        workers_soft_limit = calc_workers_soft_limit(workers_soft_limit, workers_hard_limit);
         // Create the global market instance
         size_t size = sizeof(market);
 #if __TBB_TASK_GROUP_CONTEXT
         __TBB_ASSERT( __TBB_offsetof(market, my_workers) + sizeof(generic_scheduler*) == sizeof(market),
                       "my_workers must be the last data field of the market class");
-        size += sizeof(generic_scheduler*) * (max_num_workers - 1);
+        size += sizeof(generic_scheduler*) * (workers_hard_limit - 1);
 #endif /* __TBB_TASK_GROUP_CONTEXT */
         __TBB_InitOnce::add_ref();
         void* storage = NFS_Allocate(1, size, NULL);
         memset( storage, 0, size );
         // Initialize and publish global market
-        m = new (storage) market( max_num_workers, stack_size );
+        m = new (storage) market( workers_soft_limit, workers_hard_limit, stack_size );
+        if( is_public )
+            m->my_public_ref_count = 1;
         theMarket = m;
+        // This check relies on the fact that for shared RML default_concurrency==max_concurrency
+        if ( !governor::UsePrivateRML && m->my_server->default_concurrency() < workers_soft_limit )
+            runtime_warning( "RML might limit the number of workers to %u while %u is requested.\n"
+                    , m->my_server->default_concurrency(), workers_soft_limit );
     }
     return *m;
 }
@@ -132,11 +163,15 @@ void market::destroy () {
     __TBB_InitOnce::remove_ref();
 }
 
-void market::release () {
+void market::release ( bool is_public ) {
     __TBB_ASSERT( theMarket == this, "Global market instance was destroyed prematurely?" );
     bool do_release = false;
     {
         global_market_mutex_type::scoped_lock lock(theMarketMutex);
+        if ( is_public ) {
+            __TBB_ASSERT( my_public_ref_count, NULL );
+            --my_public_ref_count;
+        }
         if ( --my_ref_count == 0 ) {
             do_release = true;
             theMarket = NULL;
@@ -148,35 +183,71 @@ void market::release () {
 
 void market::wait_workers () {
     // usable for this kind of scheduler only
-    __TBB_ASSERT(governor::needsWaitWorkers(), NULL);
+    __TBB_ASSERT(join_workers, NULL);
+    // to guarantee that request_close_connection() is called by master,
     // wait till terminating last worker decresed my_ref_count
     while (__TBB_load_with_acquire(my_ref_count) > 1)
         __TBB_Yield();
     __TBB_ASSERT(1 == my_ref_count, NULL);
-    release();
 }
 
-arena& market::create_arena ( unsigned max_num_workers, size_t stack_size ) {
-    market &m = global_market( max_num_workers, stack_size ); // increases market's ref count
-#if __TBB_TASK_ARENA
-    // Prevent cutting an extra slot for task_arena(p,0) with default market (p-1 worketrs).
-    // This is a temporary workaround for 1968 until (TODO:) master slot reservation is reworked
-    arena& a = arena::allocate_arena( m, min(max_num_workers, m.my_max_num_workers+1) );
-#else
-    arena& a = arena::allocate_arena( m, min(max_num_workers, m.my_max_num_workers) );
+void market::set_active_num_workers ( unsigned soft_limit ) {
+    int old_requested, requested;
+    market *m;
+    // at least 1 worker is required to support starvation resistant tasks
+    if( soft_limit == 0 )
+        soft_limit = 1;
+
+    {
+        global_market_mutex_type::scoped_lock lock( theMarketMutex );
+        if ( !theMarket )
+            return; // actual value will be used at market creation
+        m = theMarket;
+        ++m->my_ref_count;
+    }
+    // have my_ref_count for market, use it safely
+    {
+        arenas_list_mutex_type::scoped_lock lock(m->my_arenas_list_mutex);
+        __TBB_ASSERT(soft_limit <= m->my_num_workers_hard_limit, NULL);
+        m->my_num_workers_soft_limit = soft_limit;
+        requested = min(m->my_total_demand, (int)soft_limit);
+        old_requested = m->my_num_workers_requested;
+        m->my_num_workers_requested = requested;
+        m->my_priority_levels[m->my_global_top_priority].workers_available = soft_limit;
+        m->update_allotment(
+#if __TBB_TASK_PRIORITY
+            m->my_global_top_priority
 #endif
+            );
+    }
+    // Must be called outside of any locks
+    if ( requested != old_requested )
+        m->my_server->adjust_job_count_estimate( requested - old_requested );
+    // release internal market reference to match ++m->my_ref_count above
+    m->release();
+}
+
+bool governor::does_client_join_workers (const tbb::internal::rml::tbb_client &client) {
+    return ((const market&)client).must_join_workers();
+}
+
+arena* market::create_arena ( int num_slots, int num_reserved_slots, size_t stack_size, bool default_concurrency_requested ) {
+    __TBB_ASSERT( num_slots > 0, NULL );
+    __TBB_ASSERT( num_reserved_slots <= num_slots, NULL );
+    // Add public market reference for master thread/task_arena (that adds an internal reference in exchange).
+    market &m = global_market( /*is_public=*/true, num_slots-num_reserved_slots, stack_size,
+                               default_concurrency_requested ); 
+
+    arena& a = arena::allocate_arena( m, num_slots, num_reserved_slots );
     // Add newly created arena into the existing market's list.
     arenas_list_mutex_type::scoped_lock lock(m.my_arenas_list_mutex);
     m.insert_arena_into_list(a);
-    return a;
+    return &a;
 }
 
 /** This method must be invoked under my_arenas_list_mutex. **/
 void market::detach_arena ( arena& a ) {
     __TBB_ASSERT( theMarket == this, "Global market instance was destroyed prematurely?" );
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    __TBB_ASSERT( !a.my_num_workers_present, NULL );
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
     __TBB_ASSERT( !a.my_slots[0].my_scheduler, NULL );
     remove_arena_from_list(a);
     if ( a.my_aba_epoch == my_arenas_aba_epoch )
@@ -184,7 +255,9 @@ void market::detach_arena ( arena& a ) {
 }
 
 void market::try_destroy_arena ( arena* a, uintptr_t aba_epoch ) {
-    __TBB_ASSERT ( a, NULL );
+    // master thread holds reference to the market, so it cannot be destroyed at any moment here
+    __TBB_ASSERT( this == theMarket, NULL );
+    __TBB_ASSERT( a, NULL );
     arenas_list_mutex_type::scoped_lock lock(my_arenas_list_mutex);
     assert_market_valid();
 #if __TBB_TASK_PRIORITY
@@ -213,45 +286,20 @@ void market::try_destroy_arena ( arena* a, uintptr_t aba_epoch ) {
 #endif /* __TBB_TASK_PRIORITY */
 }
 
-void market::try_destroy_arena ( market* m, arena* a, uintptr_t aba_epoch, bool master ) {
-    // Arena may have been orphaned. Or it may have been destroyed.
-    // Thus we cannot dereference the pointer to it until its liveness is verified.
-    // Arena is alive if it is found in the market's list.
-
-    if ( m != theMarket ) {
-        // The market has already been emptied.
-        return;
-    }
-    else if ( master ) {
-        // If this is a master thread, market can be destroyed at any moment.
-        // So protect it with an extra refcount.
-        global_market_mutex_type::scoped_lock lock(theMarketMutex);
-        if ( m != theMarket )
-            return;
-        ++m->my_ref_count;
-    }
-    m->try_destroy_arena( a, aba_epoch );
-    if ( master )
-        m->release();
-}
-
 /** This method must be invoked under my_arenas_list_mutex. **/
-arena* market::arena_in_need ( arena_list_type &arenas, arena_list_type::iterator& next ) {
+arena* market::arena_in_need ( arena_list_type &arenas, arena *&next ) {
     if ( arenas.empty() )
         return NULL;
-    __TBB_ASSERT( next != arenas.end(), NULL );
     arena_list_type::iterator it = next;
+    __TBB_ASSERT( it != arenas.end(), NULL );
     do {
         arena& a = *it;
         if ( ++it == arenas.end() )
             it = arenas.begin();
         if ( a.num_workers_active() < a.my_num_workers_allotted ) {
-            a.my_references+=2; // add a worker
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-            ++a.my_num_workers_present;
-            ++my_priority_levels[a.my_top_priority].workers_present;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
-            next = it;
+            a.my_references += arena::ref_worker;
+            as_atomic(next) = &*it; // a subject for innocent data race under the reader lock
+            // TODO: rework global round robin policy to local or random to avoid this write
             return &a;
         }
     } while ( it != next );
@@ -288,49 +336,29 @@ void market::update_allotment ( arena_list_type& arenas, int workers_demand, int
 inline void market::update_global_top_priority ( intptr_t newPriority ) {
     GATHER_STATISTIC( ++governor::local_scheduler_if_initialized()->my_counters.market_prio_switches );
     my_global_top_priority = newPriority;
-    my_priority_levels[newPriority].workers_available = my_max_num_workers;
+    my_priority_levels[newPriority].workers_available = my_num_workers_soft_limit;
     advance_global_reload_epoch();
 }
 
 inline void market::reset_global_priority () {
     my_global_bottom_priority = normalized_normal_priority;
     update_global_top_priority(normalized_normal_priority);
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    my_lowest_populated_level = normalized_normal_priority;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
 }
 
-arena* market::arena_in_need (
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-                              arena* prev_arena
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
-                             )
+arena* market::arena_in_need ( arena* prev_arena )
 {
-    arenas_list_mutex_type::scoped_lock lock(my_arenas_list_mutex);
+    suppress_unused_warning(prev_arena);
+    if( as_atomic(my_total_demand) <= 0 )
+        return NULL;
+    arenas_list_mutex_type::scoped_lock lock(my_arenas_list_mutex, /*is_writer=*/false);
     assert_market_valid();
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    if ( prev_arena ) {
-        priority_level_info &pl = my_priority_levels[prev_arena->my_top_priority];
-        --prev_arena->my_num_workers_present;
-        --pl.workers_present;
-        if ( !--prev_arena->my_references && !prev_arena->my_num_workers_requested ) {
-            detach_arena( *a );
-            lock.release();
-            a->free_arena();
-            lock.acquire();
-        }
-    }
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
     int p = my_global_top_priority;
     arena *a = NULL;
     do {
         priority_level_info &pl = my_priority_levels[p];
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-        __TBB_ASSERT( p >= my_lowest_populated_level, NULL );
-        if ( pl.workers_present >= pl.workers_requested )
-            continue;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
         a = arena_in_need( pl.arenas, pl.next_arena );
+        // TODO: When refactoring task priority code, take into consideration the
+        // __TBB_TRACK_PRIORITY_LEVEL_SATURATION sections from earlier versions of TBB
     } while ( !a && --p >= my_global_bottom_priority );
     return a;
 }
@@ -338,9 +366,6 @@ arena* market::arena_in_need (
 void market::update_allotment ( intptr_t highest_affected_priority ) {
     intptr_t i = highest_affected_priority;
     int available = my_priority_levels[i].workers_available;
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    my_lowest_populated_level = my_global_bottom_priority;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
     for ( ; i >= my_global_bottom_priority; --i ) {
         priority_level_info &pl = my_priority_levels[i];
         pl.workers_available = available;
@@ -349,9 +374,6 @@ void market::update_allotment ( intptr_t highest_affected_priority ) {
             available -= pl.workers_requested;
             if ( available < 0 ) {
                 available = 0;
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-                my_lowest_populated_level = i;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
                 break;
             }
         }
@@ -362,7 +384,7 @@ void market::update_allotment ( intptr_t highest_affected_priority ) {
         pl.workers_available = 0;
         arena_list_type::iterator it = pl.arenas.begin();
         for ( ; it != pl.arenas.end(); ++it ) {
-            __TBB_ASSERT( it->my_num_workers_requested || !it->my_num_workers_allotted, NULL );
+            __TBB_ASSERT( it->my_num_workers_requested >= 0 || !it->my_num_workers_allotted, NULL );
             it->my_num_workers_allotted = 0;
         }
     }
@@ -384,23 +406,17 @@ void market::adjust_demand ( arena& a, int delta ) {
         }
         delta = -prev_req;
     }
-#if __TBB_TASK_ARENA
     else if ( prev_req < 0 ) {
         delta = a.my_num_workers_requested;
     }
-#else  /* __TBB_TASK_ARENA */
-    __TBB_ASSERT( prev_req >= 0, "Part-size request to RML?" );
-#endif /* __TBB_TASK_ARENA */
-#if __TBB_TASK_PRIORITY
+    my_total_demand += delta;
+#if !__TBB_TASK_PRIORITY
+    update_allotment();
+#else /* !__TBB_TASK_PRIORITY */
     intptr_t p = a.my_top_priority;
     priority_level_info &pl = my_priority_levels[p];
     pl.workers_requested += delta;
     __TBB_ASSERT( pl.workers_requested >= 0, NULL );
-#if !__TBB_TASK_ARENA
-    __TBB_ASSERT( a.my_num_workers_requested >= 0, NULL );
-#else
-    //TODO: understand the assertion and modify
-#endif
     if ( a.my_num_workers_requested <= 0 ) {
         if ( a.my_top_priority != normalized_normal_priority ) {
             GATHER_STATISTIC( ++governor::local_scheduler_if_initialized()->my_counters.arena_prio_resets );
@@ -420,14 +436,12 @@ void market::adjust_demand ( arena& a, int delta ) {
         update_allotment( my_global_top_priority );
     }
     else if ( p > my_global_top_priority ) {
-#if !__TBB_TASK_ARENA
         __TBB_ASSERT( pl.workers_requested > 0, NULL );
-#else
-        //TODO: understand the assertion and modify
-#endif
+        // TODO: investigate if the following invariant is always valid
+        __TBB_ASSERT( a.my_num_workers_requested >= 0, NULL );
         update_global_top_priority(p);
-        a.my_num_workers_allotted = min( (int)my_max_num_workers, a.my_num_workers_requested );
-        my_priority_levels[p - 1].workers_available = my_max_num_workers - a.my_num_workers_allotted;
+        a.my_num_workers_allotted = min( (int)my_num_workers_soft_limit, a.my_num_workers_requested );
+        my_priority_levels[p - 1].workers_available = my_num_workers_soft_limit - a.my_num_workers_allotted;
         update_allotment( p - 1 );
     }
     else if ( p == my_global_bottom_priority ) {
@@ -436,12 +450,8 @@ void market::adjust_demand ( arena& a, int delta ) {
                 continue;
             if ( p > my_global_top_priority )
                 reset_global_priority();
-            else {
+            else
                 my_global_bottom_priority = p;
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-                my_lowest_populated_level = max( my_lowest_populated_level, p );
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
-            }
         }
         else
             update_allotment( p );
@@ -458,10 +468,20 @@ void market::adjust_demand ( arena& a, int delta ) {
     }
     __TBB_ASSERT( my_global_top_priority >= a.my_top_priority || a.my_num_workers_requested<=0, NULL );
     assert_market_valid();
-#else /* !__TBB_TASK_PRIORITY */
-    my_total_demand += delta;
-    update_allotment();
 #endif /* !__TBB_TASK_PRIORITY */
+    if ( delta > 0 ) {
+        // can't overflow soft_limit, but remember values request by arenas in
+        // my_total_demand to not prematurely release workers to RML
+        if ( my_num_workers_requested+delta > (int)my_num_workers_soft_limit ) {
+            delta = my_num_workers_soft_limit - my_num_workers_requested;
+        }
+    } else
+        // the number of workers should not be decreased below my_total_demand
+        if ( my_num_workers_requested+delta < my_total_demand )
+            delta = min(my_total_demand, (int)my_num_workers_soft_limit) - my_num_workers_requested;
+    my_num_workers_requested += delta;
+    __TBB_ASSERT( my_num_workers_requested <= (int)my_num_workers_soft_limit, NULL );
+
     my_arenas_list_mutex.unlock();
     // Must be called outside of any locks
     my_server->adjust_job_count_estimate( delta );
@@ -470,14 +490,43 @@ void market::adjust_demand ( arena& a, int delta ) {
 
 void market::process( job& j ) {
     generic_scheduler& s = static_cast<generic_scheduler&>(j);
-    __TBB_ASSERT( governor::is_set(&s), NULL );
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
     arena *a = NULL;
-    while ( (a = arena_in_need(a)) )
+    __TBB_ASSERT( governor::is_set(&s), NULL );
+    enum {
+        query_interval = 1000,
+        first_interval = 1
+    };
+    for(int i = first_interval; ; i--) {
+        while ( (a = arena_in_need(a)) )
+        {
+            a->process(s);
+            i = first_interval;
+        }
+        // Workers leave market because there is no arena in need. It can happen earlier than
+        // adjust_job_count_estimate() decreases my_slack and RML can put this thread to sleep.
+        // It might result in a busy-loop checking for my_slack<0 and calling this method instantly.
+        // first_interval>0 and the pause refines this spinning.
+        if( i > 0 )
+            prolonged_pause();
+        else
+#if !__TBB_SLEEP_PERMISSION
+            break;
 #else
-    while ( arena *a = arena_in_need() )
-#endif
-        a->process(s);
+        { // i == 0
+#if __TBB_TASK_PRIORITY
+            arena_list_type &al = my_priority_levels[my_global_top_priority].arenas;
+#else /* __TBB_TASK_PRIORITY */
+            arena_list_type &al = my_arenas;
+#endif /* __TBB_TASK_PRIORITY */
+            if( al.empty() ) // races if any are innocent TODO: replace by an RML query interface
+                break; // no arenas left, perhaps going to shut down
+            if( the_global_observer_list.ask_permission_to_leave() )
+                break; // go sleep
+            __TBB_Yield();
+            i = query_interval;
+        }
+#endif// !__TBB_SLEEP_PERMISSION
+    }
     GATHER_STATISTIC( ++s.my_counters.market_roundtrips );
 }
 
@@ -485,7 +534,7 @@ void market::cleanup( job& j ) {
     __TBB_ASSERT( theMarket != this, NULL );
     generic_scheduler& s = static_cast<generic_scheduler&>(j);
     generic_scheduler* mine = governor::local_scheduler_if_initialized();
-    __TBB_ASSERT( !mine || mine->my_arena_index!=0, NULL );
+    __TBB_ASSERT( !mine || mine->is_worker(), NULL );
     if( mine!=&s ) {
         governor::assume_scheduler( &s );
         generic_scheduler::cleanup_worker( &s, mine!=NULL );
@@ -500,16 +549,16 @@ void market::acknowledge_close_connection() {
 }
 
 ::rml::job* market::create_one_job() {
-    unsigned index = ++my_num_workers;
+    unsigned index = ++my_first_unused_worker_idx;
     __TBB_ASSERT( index > 0, NULL );
     ITT_THREAD_SET_NAME(_T("TBB Worker Thread"));
     // index serves as a hint decreasing conflicts between workers when they migrate between arenas
     generic_scheduler* s = generic_scheduler::create_worker( *this, index );
 #if __TBB_TASK_GROUP_CONTEXT
+    __TBB_ASSERT( index <= my_num_workers_hard_limit, NULL );
     __TBB_ASSERT( !my_workers[index - 1], NULL );
     my_workers[index - 1] = s;
 #endif /* __TBB_TASK_GROUP_CONTEXT */
-    governor::sign_on(s);
     return s;
 }
 
@@ -522,14 +571,7 @@ void market::update_arena_top_priority ( arena& a, intptr_t new_priority ) {
     remove_arena_from_list(a);
     a.my_top_priority = new_priority;
     insert_arena_into_list(a);
-    ++a.my_reload_epoch;
-#if __TBB_TRACK_PRIORITY_LEVEL_SATURATION
-    // Arena's my_num_workers_present may remain positive for some time after its
-    // my_num_workers_requested becomes zero. Thus the following two lines are
-    // executed unconditionally.
-    prev_level.workers_present -= a.my_num_workers_present;
-    new_level.workers_present += a.my_num_workers_present;
-#endif /* __TBB_TRACK_PRIORITY_LEVEL_SATURATION */
+    ++a.my_reload_epoch; // TODO: synch with global reload epoch in order to optimize usage of local reload epoch
     prev_level.workers_requested -= a.my_num_workers_requested;
     new_level.workers_requested += a.my_num_workers_requested;
     __TBB_ASSERT( prev_level.workers_requested >= 0 && new_level.workers_requested >= 0, NULL );
@@ -566,6 +608,7 @@ bool market::lower_arena_priority ( arena& a, intptr_t new_priority, uintptr_t o
 }
 
 bool market::update_arena_priority ( arena& a, intptr_t new_priority ) {
+    // TODO: do not acquire this global lock while checking arena's state.
     arenas_list_mutex_type::scoped_lock lock(my_arenas_list_mutex);
 
     __TBB_ASSERT( my_global_top_priority >= a.my_top_priority || a.my_num_workers_requested <= 0, NULL );
@@ -623,15 +666,5 @@ bool market::update_arena_priority ( arena& a, intptr_t new_priority ) {
 }
 #endif /* __TBB_TASK_PRIORITY */
 
-#if __TBB_COUNT_TASK_NODES 
-intptr_t market::workers_task_node_count() {
-    intptr_t result = 0;
-    ForEachArena(a) {
-        result += a.workers_task_node_count();
-    } EndForEach();
-    return result;
-}
-#endif /* __TBB_COUNT_TASK_NODES */
-
 } // namespace internal
 } // namespace tbb
