@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef _TBB_observer_proxy_H
@@ -39,15 +31,12 @@
 namespace tbb {
 namespace internal {
 
-class arena;
-class observer_proxy;
-
 class observer_list {
     friend class arena;
 
     // Mutex is wrapped with aligned_space to shut up warnings when its destructor
     // is called while threads are still using it.
-    typedef aligned_space<spin_rw_mutex,1>  my_mutex_type;
+    typedef aligned_space<spin_rw_mutex>  my_mutex_type;
 
     //! Pointer to the head of this list.
     observer_proxy* my_head;
@@ -102,9 +91,9 @@ public:
     inline void notify_entry_observers( observer_proxy*& last, bool worker );
 
     //! Call exit notifications on last and observers added before it.
-    inline void notify_exit_observers( observer_proxy* last, bool worker );
+    inline void notify_exit_observers( observer_proxy*& last, bool worker );
 
-    //! Call on_scheduler_leaving callbacks to ask for permission for a worker thread to leave an arena
+    //! Call may_sleep callbacks to ask for permission for a worker thread to leave market
     bool ask_permission_to_leave();
 }; // class observer_list
 
@@ -117,7 +106,7 @@ class observer_proxy {
     friend class observer_list;
     //! Reference count used for garbage collection.
     /** 1 for reference from my task_scheduler_observer.
-        1 for each task dispatcher's last observer pointer. 
+        1 for each task dispatcher's last observer pointer.
         No accounting for neighbors in the shared list. */
     atomic<int> my_ref_count;
     //! Reference to the list this observer belongs to.
@@ -133,8 +122,12 @@ class observer_proxy {
     //! Version
     char my_version;
 
+#if __TBB_ARENA_OBSERVER || __TBB_SLEEP_PERMISSION
     interface6::task_scheduler_observer* get_v6_observer();
+#endif
+#if __TBB_ARENA_OBSERVER
     bool is_global(); //TODO: move them back inline when un-CPF'ing
+#endif
 
     //! Constructs proxy for the given observer and adds it to the specified list.
     observer_proxy( task_scheduler_observer_v3& );
@@ -149,10 +142,9 @@ class observer_proxy {
 
 inline void observer_list::remove_ref_fast( observer_proxy*& p ) {
     if( p->my_observer ) {
-        // 2 = 1 for observer and 1 for last
-        __TBB_ASSERT( p->my_ref_count>=2, NULL );
         // Can decrement refcount quickly, as it cannot drop to zero while under the lock.
-        --p->my_ref_count;
+        int r = --p->my_ref_count;
+        __TBB_ASSERT_EX( r, NULL );
         p = NULL;
     } else {
         // Use slow form of refcount decrementing, after the lock is released.
@@ -165,13 +157,16 @@ inline void observer_list::notify_entry_observers( observer_proxy*& last, bool w
     do_notify_entry_observers( last, worker );
 }
 
-inline void observer_list::notify_exit_observers( observer_proxy* last, bool worker ) {
+inline void observer_list::notify_exit_observers( observer_proxy*& last, bool worker ) {
     if ( !last )
         return;
+    __TBB_ASSERT(is_alive((uintptr_t)last), NULL);
     do_notify_exit_observers( last, worker );
+    __TBB_ASSERT(last, NULL);
+    poison_value(last);
 }
 
-extern observer_list the_global_observer_list;
+extern padded<observer_list> the_global_observer_list;
 
 } // namespace internal
 } // namespace tbb
