@@ -241,7 +241,11 @@ inline bool CancellationInfoPresent ( task& t ) {
 
 #if TBB_USE_CAPTURED_EXCEPTION
     inline tbb_exception* TbbCurrentException( task_group_context*, tbb_exception* src) { return src->move(); }
-    inline tbb_exception* TbbCurrentException( task_group_context*, captured_exception* src) { return src; }
+    inline tbb_exception* TbbCurrentException( task_group_context* c, captured_exception* src) {
+        if( c->my_version_and_traits & task_group_context::exact_exception )
+            runtime_warning( "Exact exception propagation is requested by application but the linked library is built without support for it");
+        return src;
+    }
 #else
     // Using macro instead of an inline function here allows to avoid evaluation of the
     // TbbCapturedException expression when exact propagation is enabled for the context.
@@ -272,6 +276,35 @@ inline bool ConcurrentWaitsEnabled ( task& t ) { return false; }
 
 #endif /* __TBB_TASK_GROUP_CONTEXT */
 
+inline void prolonged_pause() {
+#if defined(__TBB_time_stamp) && !__TBB_STEALING_PAUSE
+    // Assumption based on practice: 1000-2000 ticks seems to be a suitable invariant for the
+    // majority of platforms. Currently, skip platforms that define __TBB_STEALING_PAUSE
+    // because these platforms require very careful tuning.
+    machine_tsc_t prev = __TBB_time_stamp();
+    const machine_tsc_t finish = prev + 1000;
+    atomic_backoff backoff;
+    do {
+        backoff.bounded_pause();
+        machine_tsc_t curr = __TBB_time_stamp();
+        if ( curr <= prev )
+            // Possibly, the current logical thread is moved to another hadware thread or overflow is occured.
+            break;
+        prev = curr;
+    } while ( prev < finish );
+#else
+#ifdef __TBB_STEALING_PAUSE
+    static const long PauseTime = __TBB_STEALING_PAUSE;
+#elif __TBB_ipf
+    static const long PauseTime = 1500;
+#else
+    static const long PauseTime = 80;
+#endif
+    // TODO IDEA: Update PauseTime adaptively?
+    __TBB_Pause(PauseTime);
+#endif
+}
+
 //------------------------------------------------------------------------
 // arena_slot
 //------------------------------------------------------------------------
