@@ -1,32 +1,25 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "tbb/tbb_config.h"
+#include "tbb/global_control.h"
 #include "tbb_main.h"
 #include "governor.h"
 #include "market.h"
@@ -46,17 +39,15 @@ static const char _pad[NFS_MaxLineSize - sizeof(int)] = {};
 
 //------------------------------------------------------------------------
 // governor data
-basic_tls<generic_scheduler*> governor::theTLS;
+basic_tls<uintptr_t> governor::theTLS;
 unsigned governor::DefaultNumberOfThreads;
 rml::tbb_factory governor::theRMLServerFactory;
 bool governor::UsePrivateRML;
 const task_scheduler_init *governor::BlockingTSI;
 #if TBB_USE_ASSERT
-bool governor::IsBlockingTermiantionInProgress;
+bool governor::IsBlockingTerminationInProgress;
 #endif
-#if __TBB_CPF_BUILD || TBB_PREVIEW_SPECULATIVE_SPIN_RW_MUTEX
 bool governor::is_speculation_enabled;
-#endif
 
 //------------------------------------------------------------------------
 // market data
@@ -87,7 +78,7 @@ bool __TBB_InitOnce::InitializationDone;
 // generic_scheduler data
 
 //! Pointer to the scheduler factory function
-generic_scheduler* (*AllocateSchedulerPtr)( arena*, size_t index );
+generic_scheduler* (*AllocateSchedulerPtr)( market& );
 
 #if __TBB_OLD_PRIMES_RNG
 //! Table of primes used by fast random-number generator (FastRandom).
@@ -134,18 +125,12 @@ void __TBB_InitOnce::add_ref() {
         governor::acquire_resources();
 }
 
-#if __TBB_ITT_STRUCTURE_API
-static inline void ITT_fini() { }
-#endif
-
 void __TBB_InitOnce::remove_ref() {
     int k = --count;
     __TBB_ASSERT(k>=0,"removed __TBB_InitOnce ref that was not added?"); 
     if( k==0 ) {
         governor::release_resources();
-#if __TBB_ITT_STRUCTURE_API
-        ITT_fini();
-#endif
+        ITT_FINI_ITTLIB();
     }
 }
 
@@ -232,6 +217,7 @@ void ITT_DoOneTimeInitialization() {
 
 //! Performs thread-safe lazy one-time general TBB initialization.
 void DoOneTimeInitializations() {
+    suppress_unused_warning(_pad);
     __TBB_InitOnce::lock();
     // No fence required for load of InitializationDone, because we are inside a critical section.
     if( !__TBB_InitOnce::InitializationDone ) {
@@ -364,7 +350,7 @@ void itt_relation_add_v7( itt_domain_enum domain, void *addr0, unsigned long lon
 }
 
 void itt_task_begin_v7( itt_domain_enum domain, void *task, unsigned long long task_extra, 
-                        void *parent, unsigned long long parent_extra, string_index /* name_index */ ) {
+                        void *parent, unsigned long long parent_extra, string_index name_index ) {
     if ( __itt_domain *d = get_itt_domain( domain ) ) {
         __itt_id task_id = itt_null_id;
         __itt_id parent_id = itt_null_id;
@@ -372,7 +358,8 @@ void itt_task_begin_v7( itt_domain_enum domain, void *task, unsigned long long t
         if ( parent ) {
             itt_id_make( &parent_id, parent, parent_extra );
         }
-        ITTNOTIFY_VOID_D3(task_begin, d, task_id, parent_id, NULL );
+        __itt_string_handle *n = ITT_get_string_handle(name_index);
+        ITTNOTIFY_VOID_D3(task_begin, d, task_id, parent_id, n );
     }
 }
 
@@ -382,6 +369,27 @@ void itt_task_end_v7( itt_domain_enum domain ) {
     }
 }
 
+void itt_region_begin_v9( itt_domain_enum domain, void *region, unsigned long long region_extra, 
+                          void *parent, unsigned long long parent_extra, string_index /* name_index */ ) {
+    if ( __itt_domain *d = get_itt_domain( domain ) ) {
+        __itt_id region_id = itt_null_id;
+        __itt_id parent_id = itt_null_id;
+        itt_id_make( &region_id, region, region_extra );
+        if ( parent ) {
+            itt_id_make( &parent_id, parent, parent_extra );
+        }
+        ITTNOTIFY_VOID_D3(region_begin, d, region_id, parent_id, NULL );
+    }
+}
+
+void itt_region_end_v9( itt_domain_enum domain, void *region, unsigned long long region_extra ) {
+    if ( __itt_domain *d = get_itt_domain( domain ) ) {
+        __itt_id region_id = itt_null_id;
+        itt_id_make( &region_id, region, region_extra );
+        ITTNOTIFY_VOID_D1( region_end, d, region_id );
+    }
+}
+
 #else // DO_ITT_NOTIFY
 
 void itt_make_task_group_v7( itt_domain_enum domain, void *group, unsigned long long group_extra, 
@@ -398,20 +406,162 @@ void itt_task_begin_v7( itt_domain_enum domain, void *task, unsigned long long t
 
 void itt_task_end_v7( itt_domain_enum domain ) { }
 
+void itt_region_begin_v9( itt_domain_enum domain, void *region, unsigned long long region_extra, 
+                          void *parent, unsigned long long parent_extra, string_index /* name_index */ ) { }
+
+void itt_region_end_v9( itt_domain_enum domain, void *region, unsigned long long region_extra ) { }
+
 #endif // DO_ITT_NOTIFY
 
 #endif // __TBB_ITT_STRUCTURE_API
 
 void* itt_load_pointer_v3( const void* src ) {
+    //TODO: replace this with __TBB_load_relaxed
     void* result = *static_cast<void*const*>(src);
     return result;
 }
 
 void itt_set_sync_name_v3( void* obj, const tchar* name) {
     ITT_SYNC_RENAME(obj, name);
-    suppress_unused_warning(obj && name);
+    suppress_unused_warning(obj, name);
 }
 
 
+class control_storage {
+    friend class tbb::interface9::global_control;
+protected:
+    size_t my_active_value;
+    atomic<global_control*> my_head;
+    spin_mutex my_list_mutex;
+
+    virtual size_t default_value() const = 0;
+    virtual void apply_active() const {}
+    virtual bool is_first_arg_preferred(size_t a, size_t b) const {
+        return a>b; // prefer max by default
+    }
+    virtual size_t active_value() const {
+        return my_head? my_active_value : default_value();
+    }
+};
+
+class allowed_parallelism_control : public padded<control_storage> {
+    virtual size_t default_value() const {
+        // current implementation can't have effective active value below 2
+        return max(2U, governor::default_num_threads());
+    }
+    virtual bool is_first_arg_preferred(size_t a, size_t b) const {
+        return a<b; // prefer min allowed parallelism
+    }
+    virtual void apply_active() const {
+        __TBB_ASSERT( my_active_value>=2, NULL );
+        // -1 to take master into account
+        market::set_active_num_workers( my_active_value-1 );
+    }
+    virtual size_t active_value() const {
+/* Reading of my_active_value is not synchronized with possible updating
+   of my_head by other thread. It's ok, as value of my_active_value became
+   not invalid, just obsolete. */
+        if (!my_head)
+            return default_value();
+        // non-zero, if market is active
+        const size_t workers = market::max_num_workers();
+        // We can't exceed market's maximal number of workers.
+        // +1 to take master into account
+        return workers? min(workers+1, my_active_value): my_active_value;
+    }
+public:
+    size_t active_value_if_present() const {
+        return my_head? my_active_value : 0;
+    }
+};
+
+class stack_size_control : public padded<control_storage> {
+    virtual size_t default_value() const {
+        return tbb::internal::ThreadStackSize;
+    }
+    virtual void apply_active() const {
+#if __TBB_WIN8UI_SUPPORT
+        __TBB_ASSERT( false, "For Windows Store* apps we must not set stack size" );
+#endif
+    }
+};
+
+static allowed_parallelism_control allowed_parallelism_ctl;
+static stack_size_control stack_size_ctl;
+
+static control_storage *controls[] = {&allowed_parallelism_ctl, &stack_size_ctl};
+
+unsigned market::app_parallelism_limit() {
+    return allowed_parallelism_ctl.active_value_if_present();
+}
+
 } // namespace internal
+
+namespace interface9 {
+
+using namespace internal;
+using namespace tbb::internal;
+
+void global_control::internal_create() {
+    __TBB_ASSERT_RELEASE( my_param < global_control::parameter_max, NULL );
+    control_storage *const c = controls[my_param];
+
+    spin_mutex::scoped_lock lock(c->my_list_mutex);
+    if (!c->my_head || c->is_first_arg_preferred(my_value, c->my_active_value)) {
+        c->my_active_value = my_value;
+        // to guarantee that apply_active() is called with current active value,
+        // calls it here and in internal_destroy() under my_list_mutex
+        c->apply_active();
+    }
+    my_next = c->my_head;
+    // publish my_head, at this point my_active_value must be valid
+    c->my_head = this;
+}
+
+void global_control::internal_destroy() {
+    global_control *prev = 0;
+
+    __TBB_ASSERT_RELEASE( my_param < global_control::parameter_max, NULL );
+    control_storage *const c = controls[my_param];
+    __TBB_ASSERT( c->my_head, NULL );
+
+    // Concurrent reading and changing global parameter is possible.
+    // In this case, my_active_value may not match current state of parameters.
+    // This is OK because:
+    // 1) my_active_value is either current or previous
+    // 2) my_active_value is current on internal_destroy leave
+    spin_mutex::scoped_lock lock(c->my_list_mutex);
+    size_t new_active = (size_t)-1, old_active = c->my_active_value;
+
+    if ( c->my_head != this )
+        new_active = c->my_head->my_value;
+    else if ( c->my_head->my_next )
+        new_active = c->my_head->my_next->my_value;
+    // if there is only one element, new_active will be set later
+    for ( global_control *curr = c->my_head; curr; prev = curr, curr = curr->my_next )
+        if ( curr == this ) {
+            if ( prev )
+                prev->my_next = my_next;
+            else
+                c->my_head = my_next;
+        } else
+            if (c->is_first_arg_preferred(curr->my_value, new_active))
+                new_active = curr->my_value;
+
+    if ( !c->my_head ) {
+        __TBB_ASSERT( new_active==(size_t)-1, NULL );
+        new_active = c->default_value();
+    }
+    if ( new_active != old_active ) {
+        c->my_active_value = new_active;
+        c->apply_active();
+    }
+}
+
+size_t global_control::active_value( int param ) {
+    __TBB_ASSERT_RELEASE( param < global_control::parameter_max, NULL );
+    return controls[param]->active_value();
+}
+
+} // tbb::interface9
 } // namespace tbb
