@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 // Source file for miscellaneous entities that are infrequently referenced by 
@@ -87,13 +79,14 @@ namespace internal {
    this large chunk of code to be placed on a cold page. */
 void handle_perror( int error_code, const char* what ) {
     char buf[256];
-    __TBB_ASSERT( strlen(what) < sizeof(buf) - 64, "Error description is too long" );
-    sprintf(buf,"%s: ",what);
-    char* end = strchr(buf,0);
-    size_t n = buf+sizeof(buf)-end;
-    strncpy( end, strerror( error_code ), n );
+#if _MSC_VER
+ #define snprintf _snprintf
+#endif
+    int written = snprintf(buf, sizeof(buf), "%s: %s", what, strerror( error_code ));
+    // On overflow, the returned value exceeds sizeof(buf) (for GLIBC) or is negative (for MSVC).
+    __TBB_ASSERT_EX( written>0 && written<(int)sizeof(buf), "Error description is too long" );
     // Ensure that buffer ends in terminator.
-    buf[sizeof(buf)-1] = 0; 
+    buf[sizeof(buf)-1] = 0;
 #if TBB_USE_EXCEPTIONS
     throw runtime_error(buf);
 #else
@@ -109,7 +102,7 @@ void handle_win_error( int error_code ) {
                     NULL, error_code, 0, buf, sizeof(buf), NULL );
 #else
 //TODO: update with right replacement for FormatMessageA
-	sprintf_s((char*)&buf, 512, "error code %d", error_code);
+    sprintf_s((char*)&buf, 512, "error code %d", error_code);
 #endif
 #if TBB_USE_EXCEPTIONS
     throw runtime_error(buf);
@@ -144,8 +137,9 @@ void throw_exception_v4 ( exception_id eid ) {
     case eid_reservation_length_error: DO_THROW( length_error, ("reservation size exceeds permitted max size") );
     case eid_invalid_key: DO_THROW( out_of_range, ("invalid key") );
     case eid_user_abort: DO_THROW( user_abort, () );
+    case eid_bad_tagged_msg_cast: DO_THROW( runtime_error, ("Illegal tagged_msg cast") );
 #if __TBB_SUPPORTS_WORKERS_WAITING_IN_TERMINATE
-    case eid_blocking_sch_init: DO_THROW( runtime_error, ("Nesting of blocking termiantion is impossible") );
+    case eid_blocking_sch_init: DO_THROW( runtime_error, ("Nesting of blocking termination is impossible") );
 #endif
     default: break;
     }
@@ -194,36 +188,35 @@ void PrintRMLVersionInfo( void* arg, const char* server_info ) {
     PrintExtraVersionInfo( server_info, (const char *)arg );
 }
 
-#if __TBB_CPF_BUILD || TBB_PREVIEW_SPECULATIVE_SPIN_RW_MUTEX
-    //! check for transaction support.
+//! check for transaction support.
 #if _MSC_VER
 #include <intrin.h> // for __cpuid
 #endif
 bool cpu_has_speculation() {
 #if __TBB_TSX_AVAILABLE
-#if (__INTEL_COMPILER || __GNUC__ || _MSC_VER)
+#if (__INTEL_COMPILER || __GNUC__ || _MSC_VER || __SUNPRO_CC)
     bool result = false;
-    const int hle_ebx_mask = 1<<4;
+    const int rtm_ebx_mask = 1<<11;
 #if _MSC_VER
     int info[4] = {0,0,0,0};
-    const int EBX = 1;
+    const int reg_ebx = 1;
     __cpuidex(info, 7, 0);
-    result = (info[EBX] & hle_ebx_mask)!=0;
-#elif __GNUC__
-    int EBX = 0;
+    result = (info[reg_ebx] & rtm_ebx_mask)!=0;
+#elif __GNUC__ || __SUNPRO_CC
+    int32_t reg_ebx = 0;
     int32_t reg_eax = 7;
     int32_t reg_ecx = 0;
     __asm__ __volatile__ ( "movl %%ebx, %%esi\n"
                            "cpuid\n"
                            "movl %%ebx, %0\n"
                            "movl %%esi, %%ebx\n"
-                           : "=a"(EBX) : "0" (reg_eax), "c" (reg_ecx) : "esi", 
+                           : "=a"(reg_ebx) : "0" (reg_eax), "c" (reg_ecx) : "esi", 
 #if __TBB_x86_64
                            "ebx",
 #endif
                            "edx"
                            );
-    result = (EBX & hle_ebx_mask)!=0 ;
+    result = (reg_ebx & rtm_ebx_mask)!=0 ;
 #endif
     return result;
 #else
@@ -233,7 +226,6 @@ bool cpu_has_speculation() {
     return false;
 #endif /* __TBB_TSX_AVAILABLE */
 }
-#endif  /* __TBB_CPF_BUILD || TBB_PREVIEW_SPECULATIVE_SPIN_RW_MUTEX */
 
 } // namespace internal
 
