@@ -1,33 +1,27 @@
 /*
-    Copyright 2005-2010 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
-// Works on windows only
-#ifdef _WIN32
+#include "tbb/tbb_config.h"
+
+#if !__TBB_WIN8UI_SUPPORT && defined(_WIN32)
+
 #define _CRT_SECURE_NO_DEPRECATE 1
 #define __TBB_NO_IMPLICIT_LINKAGE 1
 
@@ -63,7 +57,7 @@ inline bool IsInDistance(UINT_PTR addr1, UINT_PTR addr2, __int64 dist)
 
 /*
  * When inserting a probe in 64 bits process the distance between the insertion
- * point and the target may be bigger than 2^32. In this case we are using 
+ * point and the target may be bigger than 2^32. In this case we are using
  * indirect jump through memory where the offset to this memory location
  * is smaller than 2^32 and it contains the absolute address (8 bytes).
  *
@@ -138,22 +132,22 @@ MemoryBuffer *CreateBuffer(UINT_PTR addr)
     }
 
 public:
-    MemoryProvider() 
-    { 
+    MemoryProvider()
+    {
         SYSTEM_INFO sysInfo;
         GetSystemInfo(&sysInfo);
-        m_allocSize = sysInfo.dwAllocationGranularity; 
+        m_allocSize = sysInfo.dwAllocationGranularity;
         m_lastBuffer = &m_pages[0];
     }
 
     // We can't free the pages in the destructor because the trampolines
     // are using these memory locations and a replaced function might be called
     // after the destructor was called.
-    ~MemoryProvider() 
+    ~MemoryProvider()
     {
     }
 
-    // Return a memory location in distance less than 2^31 from input address 
+    // Return a memory location in distance less than 2^31 from input address
     UINT_PTR GetLocation(UINT_PTR addr)
     {
         MemoryBuffer *pBuff = m_pages;
@@ -185,23 +179,23 @@ private:
 static MemoryProvider memProvider;
 
 // Compare opcodes from dictionary (str1) and opcodes from code (str2)
-// str1 might contain '*' to mask adresses
-// RETURN: NULL if opcodes did not match, string lentgh of str1 on success
+// str1 might contain '*' to mask addresses
+// RETURN: NULL if opcodes did not match, string length of str1 on success
 size_t compareStrings( const char *str1, const char *str2 )
 {
-   size_t str1Lentgh = strlen(str1);
-   for (size_t i=0; i<str1Lentgh; i++){
+   size_t str1Length = strlen(str1);
+   for (size_t i=0; i<str1Length; i++){
        if( str1[i] != '*' && str1[i] != str2[i] ) return 0;
    }
-   return str1Lentgh;
+   return str1Length;
 }
 
 // Check function prologue with know prologues from the dictionary
 // opcodes - dictionary
 // inpAddr - pointer to function prologue
-// Dictionary contains opcodes for several full asm instrutions
+// Dictionary contains opcodes for several full asm instructions
 // + one opcode byte for the next asm instruction for safe address processing
-// RETURN: number of bytes for safe bytes replacement 
+// RETURN: number of bytes for safe bytes replacement
 // (matched_pattern/2-1)
 UINT CheckOpcodes( const char ** opcodes, void *inpAddr )
 {
@@ -216,7 +210,7 @@ UINT CheckOpcodes( const char ** opcodes, void *inpAddr )
     // max length and number of patterns
     if( !opcodesStringsCount || opcodes_pointer != (size_t)opcodes ){
         while( *(opcodes + opcodesStringsCount)!= NULL ){
-            if( (i=strlen(*(opcodes + opcodesStringsCount))) > maxOpcodesLength ) 
+            if( (i=strlen(*(opcodes + opcodesStringsCount))) > maxOpcodesLength )
                 maxOpcodesLength = i;
             opcodesStringsCount++;
         }
@@ -226,20 +220,20 @@ UINT CheckOpcodes( const char ** opcodes, void *inpAddr )
 
     // Translate prologue opcodes to string format to compare
     for( i=0; i< maxOpcodesLength/2; i++ ){
-        sprintf( opcodeString + 2*i, "%.2X", *((unsigned char*)inpAddr+i) ); 
+        sprintf( opcodeString + 2*i, "%.2X", *((unsigned char*)inpAddr+i) );
     }
     opcodeString[maxOpcodesLength] = 0;
-    
+
     // Compare translated opcodes with patterns
     for( i=0; i< opcodesStringsCount; i++ ){
         result = compareStrings( opcodes[i],opcodeString );
-        if( result ) 
+        if( result )
             return (UINT)(result/2-1);
     }
-    // TODO: to add more stuff to patterns
-    __TBB_ASSERT( false, "CheckOpcodes failed" );
+    // Impossibility to find opcodes in the dictionary is a serious issue,
+    // if we unable to call original function, that leads to leak or crash.
+    __TBB_ASSERT_RELEASE( false, "CheckOpcodes failed" );
 
-    // No matches found just do not store original calls
     return 0;
 }
 
@@ -258,8 +252,14 @@ static DWORD InsertTrampoline32(void *inpAddr, void *targetAddr, const char ** o
     UINT offset32;
     UCHAR *codePtr = (UCHAR *)inpAddr;
 
-    // If requested, store original function code
-    if ( storedAddr ){
+    if ( opcodes == PROCESS_JMP ){ // expecting JMP relative instruction "e9 00 00 00 00"
+        __TBB_ASSERT(*(char*)inpAddr == 0xE9, NULL);
+        __TBB_ASSERT(storedAddr, NULL);
+        // This is special case when system function consists of single near jump,
+        // so instead of moving it somewhere we use the target of the jump as original function.
+        unsigned offsetInJmp = *((unsigned*)((char*)inpAddr + 1));
+        *storedAddr = (void*)((uintptr_t)inpAddr + offsetInJmp + SIZE_OF_RELJUMP);
+    }else if ( storedAddr ){ // If requested, store original function code
         opcodesNumber = CheckOpcodes( opcodes, inpAddr );
         if( opcodesNumber >= SIZE_OF_RELJUMP ){
             UINT_PTR strdAddr = memProvider.GetLocation(srcAddr);
@@ -324,7 +324,14 @@ static DWORD InsertTrampoline64(void *inpAddr, void *targetAddr, const char ** o
     *locPtr = tgtAddr;
 
     // If requested, store original function code
-    if( storedAddr ){
+    if ( opcodes == PROCESS_JMP ){ // expecting JMP relative instruction "e9 00 00 00 00"
+        __TBB_ASSERT(*(char*)inpAddr == 0xE9, NULL);
+        __TBB_ASSERT(storedAddr, NULL);
+        // This is special case when system function consists of single near jump,
+        // so instead of moving it somewhere we use the target of the jump as original function.
+        unsigned offsetInJmp = *((unsigned*)((char*)inpAddr + 1));
+        *storedAddr = (void*)((uintptr_t)inpAddr + offsetInJmp + SIZE_OF_RELJUMP);
+    } else if( storedAddr ){
         opcodesNumber = CheckOpcodes( opcodes, inpAddr );
         if( opcodesNumber >= SIZE_OF_INDJUMP ){
             UINT_PTR strdAddr = memProvider.GetLocation(srcAddr);
@@ -396,7 +403,7 @@ static bool InsertTrampoline(void *inpAddr, void *targetAddr, const char ** opco
 FRR_TYPE ReplaceFunctionA(const char *dllName, const char *funcName, FUNCPTR newFunc, const char ** opcodes, FUNCPTR* origFunc)
 {
     // Cache the results of the last search for the module
-    // Assume that there was no DLL unload between 
+    // Assume that there was no DLL unload between
     static char cachedName[MAX_PATH+1];
     static HMODULE cachedHM = 0;
 
@@ -436,7 +443,7 @@ FRR_TYPE ReplaceFunctionA(const char *dllName, const char *funcName, FUNCPTR new
 FRR_TYPE ReplaceFunctionW(const wchar_t *dllName, const char *funcName, FUNCPTR newFunc, const char ** opcodes, FUNCPTR* origFunc)
 {
     // Cache the results of the last search for the module
-    // Assume that there was no DLL unload between 
+    // Assume that there was no DLL unload between
     static wchar_t cachedName[MAX_PATH+1];
     static HMODULE cachedHM = 0;
 
@@ -473,4 +480,4 @@ FRR_TYPE ReplaceFunctionW(const wchar_t *dllName, const char *funcName, FUNCPTR
     return FRR_OK;
 }
 
-#endif //_WIN32
+#endif /* !__TBB_WIN8UI_SUPPORT && defined(_WIN32) */
