@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef harness_iterator_H
@@ -99,8 +91,10 @@ public:
     T& operator* () { return *my_ptr; }
     RandomIterator& operator++ () { ++my_ptr; return *this; }
     bool operator== ( const RandomIterator& r ) { return my_ptr == r.my_ptr; }
-    difference_type operator- (const RandomIterator &r) {return my_ptr - r.my_ptr;}
+    bool operator!= ( const RandomIterator& r ) { return my_ptr != r.my_ptr; }
+    difference_type operator- (const RandomIterator &r) const {return my_ptr - r.my_ptr;}
     RandomIterator operator+ (difference_type n) {return RandomIterator(my_ptr + n);}
+    bool operator< (const RandomIterator &r) const {return my_ptr < r.my_ptr;}
 };
 
 template <class T>
@@ -124,8 +118,10 @@ public:
     const T& operator* () { return *my_ptr; }
     ConstRandomIterator& operator++ () { ++my_ptr; return *this; }
     bool operator== ( const ConstRandomIterator& r ) { return my_ptr == r.my_ptr; }
-    difference_type operator- (const ConstRandomIterator &r) {return my_ptr - r.my_ptr;}
+    bool operator!= ( const ConstRandomIterator& r ) { return my_ptr != r.my_ptr; }
+    difference_type operator- (const ConstRandomIterator &r) const {return my_ptr - r.my_ptr;}
     ConstRandomIterator operator+ (difference_type n) {return ConstRandomIterator(my_ptr + n);}
+    bool operator< (const ConstRandomIterator &r) const {return my_ptr < r.my_ptr;}
 };
 
 } // namespace Harness
