@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks. Threading Building Blocks is free software;
     you can redistribute it and/or modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 
 #include "harness_assert.h"
 #if !__TBB_SOURCE_DIRECTLY_INCLUDED
-// harness_allocator.h requires atimics. We do not want dependency 
+// harness_allocator.h requires atimics. We do not want dependency
 // to TBB library to get atomics, so add rudimentary implementation of them.
 #include "harness_tbb_independence.h"
 #endif
@@ -74,8 +74,6 @@ public:
     }
 };
 
-#if TBB_USE_EXCEPTIONS
-
 class NullAllocator {
 public:
     typedef char value_type;
@@ -88,19 +86,12 @@ public:
 
 void TestZeroSpaceMemoryPool()
 {
-    try {
-        tbb::memory_pool<NullAllocator> pool;
-        ASSERT(0, "Useless allocator with no memory must not be created");
-    } catch (std::bad_alloc) {
-    } catch (...) {
-        ASSERT(0, "wrong exception type; expected bad_alloc");
-    }
+    tbb::memory_pool<NullAllocator> pool;
+    bool allocated = pool.malloc(16) || pool.malloc(9*1024);
+    ASSERT(!allocated, "Allocator with no memory must not allocate anything.");
 }
 
-#else // TBB_USE_EXCEPTIONS
-
-void TestZeroSpaceMemoryPool() { }
-
+#if !TBB_USE_EXCEPTIONS
 struct FixedPool {
     void  *buf;
     size_t size;
@@ -115,10 +106,9 @@ static void *fixedBufGetMem(intptr_t pool_id, size_t &bytes)
 
     ((FixedPool*)pool_id)->used = true;
     bytes = ((FixedPool*)pool_id)->size;
-    return ((FixedPool*)pool_id)->buf;
+    return bytes? ((FixedPool*)pool_id)->buf : NULL;
 }
-
-#endif // TBB_USE_EXCEPTIONS
+#endif
 
 /* test that pools in small space are either usable or not created
    (i.e., exception raised) */
@@ -137,10 +127,10 @@ void TestSmallFixedSizePool()
    so it requires at least 16KB. Requirement of 9KB allocation is more modest.
 */
             allocated = pool.malloc( 16 ) || pool.malloc( 9*1024 );
-            ASSERT(allocated, "If pool created, it must be useful.");
-        } catch (std::bad_alloc) {
+        } catch (std::invalid_argument) {
+            ASSERT(!sz, "expect std::invalid_argument for zero-sized pool only");
         } catch (...) {
-            ASSERT(0, "wrong exception type; expected bad_alloc");
+            ASSERT(0, "wrong exception type;");
         }
 #else
 /* Do not test high-level pool interface because pool ctor emit exception
@@ -156,7 +146,6 @@ void TestSmallFixedSizePool()
 
         if (ret == rml::POOL_OK) {
             allocated = pool_malloc(pool, 16) || pool_malloc(pool, 9*1024);
-            ASSERT(allocated, "If pool created, it must be useful.");
             pool_destroy(pool);
         } else
             ASSERT(ret == rml::NO_MEMORY, "Expected that pool either valid "
@@ -169,9 +158,9 @@ void TestSmallFixedSizePool()
     try {
         tbb::fixed_pool pool(NULL, 10*1024*1024);
         ASSERT(0, "Useless allocator with no memory must not be created");
-    } catch (std::bad_alloc) {
+    } catch (std::invalid_argument) {
     } catch (...) {
-        ASSERT(0, "wrong exception type; expected bad_alloc");
+        ASSERT(0, "wrong exception type; expected invalid_argument");
     }
 #endif
 }
