@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_graph.h"
@@ -61,12 +53,12 @@ template<typename IO>
 struct pass_through {
     IO operator()(const IO& i) { return i; }
 };
-     
+
 template< typename InputType, typename OutputType, typename Body >
 void buffered_levels( size_t concurrency, Body body ) {
 
    // Do for lc = 1 to concurrency level
-   for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
+   for ( size_t lc = 1; lc <= concurrency; ++lc ) {
    tbb::flow::graph g;
 
    // Set the execute_counter back to zero in the harness
@@ -79,7 +71,7 @@ void buffered_levels( size_t concurrency, Body body ) {
    // Create the function_node with the appropriate concurrency level, and use default buffering
    tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, body );
    tbb::flow::function_node<InputType, InputType> pass_thru( g, tbb::flow::unlimited, pass_through<InputType>());
-   
+
    // Create a vector of identical exe_nodes and pass_thrus
    std::vector< tbb::flow::function_node< InputType, OutputType > > exe_vec(2, exe_node);
    std::vector< tbb::flow::function_node< InputType, InputType > > pass_thru_vec(2, pass_thru);
@@ -105,22 +97,22 @@ void buffered_levels( size_t concurrency, Body body ) {
             senders = new harness_counting_sender<InputType>[num_senders];
             for (size_t s = 0; s < num_senders; ++s ) {
                senders[s].my_limit = N;
-               tbb::flow::make_edge( senders[s], pass_thru_vec[node_idx] );
+               senders[s].register_successor(pass_thru_vec[node_idx] );
             }
 
             // Initialize the receivers so they know how many senders and messages to check for
             for (size_t r = 0; r < num_receivers; ++r ) {
-                 receivers[r].initialize_map( N, num_senders ); 
+                 receivers[r].initialize_map( N, num_senders );
             }
 
             // Do the test
             NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
             g.wait_for_all();
 
-            // confirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
-                ASSERT( n == N, NULL ); 
+                ASSERT( n == N, NULL );
                 ASSERT( senders[s].my_receiver == &pass_thru_vec[node_idx], NULL );
             }
             // validate the receivers
@@ -152,11 +144,12 @@ struct inc_functor {
     tbb::atomic<size_t> local_execute_count;
     inc_functor( ) { local_execute_count = 0; }
     inc_functor( const inc_functor &f ) { local_execute_count = f.local_execute_count; }
+    void operator=( const inc_functor &f ) { local_execute_count = f.local_execute_count; }
 
     int operator()( int i ) {
        ++global_execute_count;
        ++local_execute_count;
-       return i; 
+       return i;
     }
 
 };
@@ -165,13 +158,13 @@ template< typename InputType, typename OutputType >
 void buffered_levels_with_copy( size_t concurrency ) {
 
     // Do for lc = 1 to concurrency level
-    for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
+    for ( size_t lc = 1; lc <= concurrency; ++lc ) {
         tbb::flow::graph g;
 
         inc_functor cf;
         cf.local_execute_count = Offset;
         global_execute_count = Offset;
-       
+
         tbb::flow::function_node< InputType, OutputType > exe_node( g, lc, cf );
 
         for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
@@ -189,7 +182,7 @@ void buffered_levels_with_copy( size_t concurrency ) {
                 }
 
                 for (size_t r = 0; r < num_receivers; ++r ) {
-                    receivers[r].initialize_map( N, num_senders ); 
+                    receivers[r].initialize_map( N, num_senders );
                 }
 
                 NativeParallelFor( (int)num_senders, parallel_put_until_limit<InputType>(senders) );
@@ -197,7 +190,7 @@ void buffered_levels_with_copy( size_t concurrency ) {
 
                 for (size_t s = 0; s < num_senders; ++s ) {
                     size_t n = senders[s].my_received;
-                    ASSERT( n == N, NULL ); 
+                    ASSERT( n == N, NULL );
                     ASSERT( senders[s].my_receiver == &exe_node, NULL );
                 }
                 for (size_t r = 0; r < num_receivers; ++r ) {
@@ -218,10 +211,14 @@ void buffered_levels_with_copy( size_t concurrency ) {
 
         // validate that the local body matches the global execute_count and both are correct
         inc_functor body_copy = tbb::flow::copy_body<inc_functor>( exe_node );
-        const size_t expected_count = N/2 * MAX_NODES * MAX_NODES * ( MAX_NODES + 1 ) + MAX_NODES + Offset; 
+        const size_t expected_count = N/2 * MAX_NODES * MAX_NODES * ( MAX_NODES + 1 ) + MAX_NODES + Offset;
         size_t global_count = global_execute_count;
         size_t inc_count = body_copy.local_execute_count;
-        ASSERT( global_count == expected_count && global_count == inc_count, NULL ); 
+        ASSERT( global_count == expected_count && global_count == inc_count, NULL );
+        g.reset(tbb::flow::rf_reset_bodies);
+        body_copy = tbb::flow::copy_body<inc_functor>( exe_node );
+        inc_count = body_copy.local_execute_count;
+        ASSERT( Offset == inc_count, "reset(rf_reset_bodies) did not reset functor" );
     }
 }
 
@@ -239,16 +236,16 @@ void run_buffered_levels( int c ) {
 //! Performs test on executable nodes with limited concurrency
 /** Theses tests check:
     1) that the nodes will accepts puts up to the concurrency limit,
-    2) the nodes do not exceed the concurrency limit even when run with more threads (this is checked in the harness_graph_executor), 
+    2) the nodes do not exceed the concurrency limit even when run with more threads (this is checked in the harness_graph_executor),
     3) the nodes will receive puts from multiple successors simultaneously,
     and 4) the nodes will send to multiple predecessors.
     There is no checking of the contents of the messages for corruption.
 */
-     
+
 template< typename InputType, typename OutputType, typename Body >
 void concurrency_levels( size_t concurrency, Body body ) {
 
-   for ( size_t lc = 1; lc <= concurrency; ++lc ) { 
+   for ( size_t lc = 1; lc <= concurrency; ++lc ) {
    tbb::flow::graph g;
 
    // Set the execute_counter back to zero in the harness
@@ -258,15 +255,30 @@ void concurrency_levels( size_t concurrency, Body body ) {
    // Set the max allowed executors to lc. There is a check in the functor to make sure this is never exceeded.
    harness_graph_executor<InputType, OutputType>::max_executors = lc;
 
-   tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > exe_node( g, lc, body );
+   typedef tbb::flow::function_node< InputType, OutputType, tbb::flow::rejecting > fnode_type;
+   fnode_type exe_node( g, lc, body );
 
    for (size_t num_receivers = 1; num_receivers <= MAX_NODES; ++num_receivers ) {
 
         harness_counting_receiver<OutputType> *receivers = new harness_counting_receiver<OutputType>[num_receivers];
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT(exe_node.successor_count() == 0, NULL);
+        ASSERT(exe_node.predecessor_count() == 0, NULL);
+#endif
+
         for (size_t r = 0; r < num_receivers; ++r ) {
             tbb::flow::make_edge( exe_node, receivers[r] );
         }
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT(exe_node.successor_count() == num_receivers, NULL);
+        typename fnode_type::successor_list_type my_succs;
+        exe_node.copy_successors(my_succs);
+        ASSERT(my_succs.size() == num_receivers, NULL);
+        typename fnode_type::predecessor_list_type my_preds;
+        exe_node.copy_predecessors(my_preds);
+        ASSERT(my_preds.size() == 0, NULL);
+#endif
 
         harness_counting_sender<InputType> *senders = NULL;
 
@@ -293,10 +305,10 @@ void concurrency_levels( size_t concurrency, Body body ) {
             // wait for graph to settle down
             g.wait_for_all();
 
-            // confirm that each sender was requested from N times 
+            // confirm that each sender was requested from N times
             for (size_t s = 0; s < num_senders; ++s ) {
                 size_t n = senders[s].my_received;
-                ASSERT( n == N, NULL ); 
+                ASSERT( n == N, NULL );
                 ASSERT( senders[s].my_receiver == &exe_node, NULL );
             }
             // confirm that each receivers got N * num_senders + the initial lc puts
@@ -331,7 +343,7 @@ void run_concurrency_levels( int c ) {
 }
 
 
-struct empty_no_assign { 
+struct empty_no_assign {
    empty_no_assign() {}
    empty_no_assign( int ) {}
    operator int() { return 0; }
@@ -378,11 +390,11 @@ void unlimited_concurrency( Body body ) {
             }
 
             NativeParallelFor( p, parallel_puts<InputType>(exe_node) );
-            g.wait_for_all(); 
+            g.wait_for_all();
 
             // 2) the nodes will receive puts from multiple predecessors simultaneously,
             size_t ec = harness_graph_executor<InputType, OutputType>::execute_count;
-            ASSERT( (int)ec == p*N, NULL ); 
+            ASSERT( (int)ec == p*N, NULL );
             for (size_t r = 0; r < num_receivers; ++r ) {
                 size_t c = receivers[r].my_count;
                 // 3) the nodes will send to multiple successors.
@@ -402,7 +414,7 @@ void run_unlimited_concurrency() {
     unlimited_concurrency<InputType,OutputType>( typename harness_graph_executor<InputType, OutputType>::functor() );
 }
 
-struct continue_msg_to_int : private NoAssign {
+struct continue_msg_to_int {
     int my_int;
     continue_msg_to_int(int x) : my_int(x) {}
     int operator()(tbb::flow::continue_msg) { return my_int; }
@@ -416,10 +428,10 @@ void test_function_node_with_continue_msg_as_input() {
 
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN1( g, tbb::flow::serial, continue_msg_to_int(42));
     tbb::flow::function_node<tbb::flow::continue_msg, int, tbb::flow::rejecting> FN2( g, tbb::flow::serial, continue_msg_to_int(43));
-    
+
     tbb::flow::make_edge( Start, FN1 );
     tbb::flow::make_edge( Start, FN2 );
-    
+
     Start.try_put( tbb::flow::continue_msg() );
     g.wait_for_all();
 }
@@ -439,7 +451,114 @@ void test_concurrency(int num_threads) {
     test_function_node_with_continue_msg_as_input();
 }
 
-int TestMain() { 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+struct add_to_counter {
+    int* counter;
+    add_to_counter(int& var):counter(&var){}
+    int operator()(int i){*counter+=1; return i + 1;}
+};
+
+template<typename FTYPE>
+void test_extract() {
+    int my_count = 0;
+    int cm;
+    tbb::flow::graph g;
+    tbb::flow::broadcast_node<int> b0(g);
+    tbb::flow::broadcast_node<int> b1(g);
+    tbb::flow::function_node<int, int, FTYPE> f0(g, tbb::flow::unlimited, add_to_counter(my_count));
+    tbb::flow::queue_node<int> q0(g);
+
+    tbb::flow::make_edge(b0, f0);
+    tbb::flow::make_edge(b1, f0);
+    tbb::flow::make_edge(f0, q0);
+    for( int i = 0; i < 2; ++i ) {
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 1, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 2 && f0.successor_count() == 1, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 1 && q0.successor_count() == 0, "q0 has incorrect counts");
+
+        /* b0         */
+        /*   \        */
+        /*    f0 - q0 */
+        /*   /        */
+        /* b1         */
+
+        b0.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 1, "function_node didn't fire");
+        ASSERT(q0.try_get(cm), "function_node didn't forward");
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 2, "function_node didn't fire");
+        ASSERT(q0.try_get(cm), "function_node didn't forward");
+
+        b0.extract();
+
+        /* b0         */
+        /*            */
+        /*    f0 - q0 */
+        /*   /        */
+        /* b1         */
+
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 1, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 1, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 1 && q0.successor_count() == 0, "q0 has incorrect counts");
+        b0.try_put(1);
+        b0.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 2, "b0 messages being forwarded to function_node even though it is disconnected");
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
+        ASSERT(q0.try_get(cm), "function_node didn't forward second time");
+
+        f0.extract();
+
+        /* b0         */
+        /*            */
+        /*    f0   q0 */
+        /*            */
+        /* b1         */
+
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 0, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 0 && f0.successor_count() == 0, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 0 && q0.successor_count() == 0, "q0 has incorrect counts");
+        b0.try_put(1);
+        b0.try_put(1);
+        b1.try_put(1);
+        b1.try_put(1);
+        g.wait_for_all();
+        ASSERT(my_count == 3, "function_node didn't fire though it has only one predecessor");
+        ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
+        make_edge(b0, f0);
+
+        /* b0         */
+        /*   \        */
+        /*    f0   q0 */
+        /*            */
+        /* b1         */
+
+        ASSERT(b0.predecessor_count() == 0 && b0.successor_count() == 1, "b0 has incorrect counts");
+        ASSERT(b1.predecessor_count() == 0 && b1.successor_count() == 0, "b1 has incorrect counts");
+        ASSERT(f0.predecessor_count() == 1 && f0.successor_count() == 0, "f0 has incorrect counts");
+        ASSERT(q0.predecessor_count() == 0 && q0.successor_count() == 0, "q0 has incorrect counts");
+
+        b0.try_put(int());
+        g.wait_for_all();
+
+        ASSERT(my_count == 4, "function_node didn't fire though it has only one predecessor");
+        ASSERT(!q0.try_get(cm), "function_node forwarded though it shouldn't");
+
+        tbb::flow::make_edge(b1, f0);
+        tbb::flow::make_edge(f0, q0);
+        my_count = 0;
+    }
+}
+#endif
+
+int TestMain() {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -447,6 +566,11 @@ int TestMain() {
     for( int p=MinThread; p<=MaxThread; ++p ) {
        test_concurrency(p);
    }
+
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    test_extract<tbb::flow::rejecting>();
+    test_extract<tbb::flow::queueing>();
+#endif
    return Harness::Done;
 }
 
