@@ -1,31 +1,34 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
+#include "harness_defs.h"
+#if __TBB_TEST_SKIP_LAMBDA
+
+#include "harness.h"
+int TestMain() {
+    REPORT("Known issue: lambdas are not properly supported on the platform \n");
+    return Harness::Skipped;
+}
+
+#else /*__TBB_TEST_SKIP_LAMBDA*/
+
 #define NOMINMAX
 #include "tbb/tbb.h"
 #include "tbb/combinable.h"
@@ -63,11 +66,9 @@ int Fib(int n) {
     } else {
         int x=0, y=0;
         task_group g;
-#if __TBB_LAMBDAS_PRESENT
         g.run( [&]{x=Fib(n-1);} ); // spawn a task
         g.run( [&]{y=Fib(n-2);} ); // spawn another task
         g.wait();                  // wait for both tasks to complete
-#endif
         return x+y;
     }
 }
@@ -77,7 +78,6 @@ int Fib(int n) {
 #include "harness_assert.h"
 
 int TestMain () {
-#if __TBB_LAMBDAS_PRESENT
     const int N = 1000;
     const int Grainsize = N/1000;
     int a[N];
@@ -211,7 +211,7 @@ int TestMain () {
                                  return std::make_pair(x.first<y.first?x.first:y.first,
                                                        x.second>y.second?x.second:y.second);
                              });
-        ASSERT(minmax_result_c.first==0 && minmax_result_c.second==999, 
+        ASSERT(minmax_result_c.first==0 && minmax_result_c.second==999,
                "combinable w/lambda failed.\n");
         REMARK("passed.\n");
 
@@ -245,7 +245,5 @@ int TestMain () {
         REMARK("passed.\n");
     }
     return Harness::Done;
-#else
-    return Harness::Skipped;
-#endif /* !__TBB_LAMBDAS_PRESENT */
 }
+#endif /* __TBB_TEST_SKIP_LAMBDA */
