@@ -37,22 +37,21 @@ struct serial_receiver : public tbb::flow::receiver<T> {
 
    /* override */ tbb::task *try_put_task( const T &v ) {
        ASSERT( next_value++  == v, NULL );
-       return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
+       return const_cast<tbb::task *>(tbb::flow::interface8::SUCCESSFULLY_ENQUEUED);
    }
 
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
     typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    built_predecessors_type bpt;
+    built_predecessors_type &built_predecessors() { return bpt; }
     void internal_add_built_predecessor( tbb::flow::sender<T> & ) { }
     void internal_delete_built_predecessor( tbb::flow::sender<T> & ) { }
     void copy_predecessors( predecessor_list_type & ) { }
     size_t predecessor_count() { return 0; }
 #endif
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
    /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {next_value = T(0);}
-#else
-   /*override*/void reset_receiver() {next_value = T(0);}
-#endif
 };
 
 template< typename T >
@@ -64,19 +63,20 @@ struct parallel_receiver : public tbb::flow::receiver<T> {
 
    /* override */ tbb::task *try_put_task( const T &/*v*/ ) {
        ++my_count;
-       return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
+       return const_cast<tbb::task *>(tbb::flow::interface8::SUCCESSFULLY_ENQUEUED);
    }
 
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
     typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    built_predecessors_type bpt;
+    built_predecessors_type &built_predecessors() { return bpt; }
     void internal_add_built_predecessor( tbb::flow::sender<T> & ) { }
     void internal_delete_built_predecessor( tbb::flow::sender<T> & ) { }
     void copy_predecessors( predecessor_list_type & ) { }
     size_t predecessor_count( ) { return 0; }
-   /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {my_count = 0;}
-#else
-   /*override*/void reset_receiver() {my_count = 0;}
 #endif
+   /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {my_count = 0;}
 };
 
 template< typename T >
@@ -84,7 +84,10 @@ struct empty_sender : public tbb::flow::sender<T> {
         /* override */ bool register_successor( tbb::flow::receiver<T> & ) { return false; }
         /* override */ bool remove_successor( tbb::flow::receiver<T> & ) { return false; }
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        typedef typename tbb::flow::sender<T>::built_successors_type built_successors_type;
         typedef typename tbb::flow::sender<T>::successor_list_type successor_list_type;
+        built_successors_type bst;
+        built_successors_type &built_successors() { return bst; }
         void    internal_add_built_successor( tbb::flow::receiver<T> & ) { }
         void internal_delete_built_successor( tbb::flow::receiver<T> & ) { }
         void copy_successors( successor_list_type & ) { }
@@ -147,7 +150,7 @@ void test_puts_with_decrements( int num_threads, tbb::flow::limiter_node< T >& l
     ASSERT(lim.decrement.predecessor_count() == 1, NULL);
     ASSERT(lim.successor_count() == 1, NULL);
     ASSERT(lim.predecessor_count() == 0, NULL);
-    typename tbb::flow::interface7::internal::decrementer<tbb::flow::limiter_node<T> >::predecessor_list_type dec_preds;
+    typename tbb::flow::interface8::internal::decrementer<tbb::flow::limiter_node<T> >::predecessor_list_type dec_preds;
     lim.decrement.copy_predecessors(dec_preds);
     ASSERT(dec_preds.size() == 1, NULL);
 #endif
@@ -411,7 +414,7 @@ void test_extract() {
     tbb::flow::broadcast_node<tbb::flow::continue_msg> b1(g);
 
     for( int i = 0; i < 2; ++i ) {
-
+        REMARK("At pass %d\n", i);
         ASSERT(node0.predecessor_count() == 0, "incorrect predecessor count at start");
         ASSERT(node0.successor_count() == 0, "incorrect successor count at start");
         ASSERT(node0.decrement.predecessor_count() == 0, "incorrect decrement pred count at start");
@@ -436,7 +439,8 @@ void test_extract() {
         ASSERT(node0.predecessor_count() == 2, "incorrect predecessor count after construction");
         ASSERT(node0.successor_count() == 1, "incorrect successor count after construction");
         ASSERT(node0.decrement.predecessor_count() == 2, "incorrect decrement pred count after construction");
-        ASSERT(q2.try_get(j) && j == i, "improper value forwarded to output queue");
+        ASSERT(q2.try_get(j), "fetch of value forwarded to output queue failed");
+        ASSERT(j == i, "improper value forwarded to output queue");
         q0.try_put(2*i);
         g.wait_for_all();
         ASSERT(!q2.try_get(j), "limiter_node forwarded item improperly");
@@ -471,6 +475,9 @@ void test_extract() {
 
         if(i == 0) {
             node0.extract();
+            ASSERT(node0.predecessor_count() == 0, "incorrect predecessor count after extraction");
+            ASSERT(node0.successor_count() == 0, "incorrect successor count after extraction");
+            ASSERT(node0.decrement.predecessor_count() == 0, "incorrect decrement pred count after extraction");
         }
         else {
             q0.extract();
