@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks. Threading Building Blocks is free software;
     you can redistribute it and/or modify it under the terms of the GNU General Public License
@@ -214,9 +214,7 @@ public:
         {
             srand((UINT)time(NULL));
             FullLog=isVerbose;
-            rand();
         }
-    void InvariantDataRealloc(bool aligned); //realloc does not change data
     void NULLReturn(UINT MinSize, UINT MaxSize, int total_threads); // NULL pointer + check errno
     void UniquePointer(); // unique pointer - check with padding
     void AddrArifm(); // unique pointer - check with pointer arithmetic
@@ -254,6 +252,10 @@ struct RoundRobin: NoAssign {
 
 bool CMemTest::firstTime = true;
 
+inline size_t choose_random_alignment() {
+    return sizeof(void*)<<(rand() % POWERS_OF_2);
+}
+
 static void setSystemAllocs()
 {
     Tmalloc=malloc;
@@ -265,7 +267,7 @@ static void setSystemAllocs()
     Raligned_realloc=_aligned_realloc;
     Taligned_free=_aligned_free;
     Rposix_memalign=0;
-#elif  __APPLE__ || __sun || __ANDROID__ 
+#elif  __APPLE__ || __sun || __ANDROID__
 // OS X*, Solaris, and Android don't have posix_memalign
     Raligned_malloc=0;
     Raligned_realloc=0;
@@ -359,6 +361,68 @@ void CheckArgumentsOverflow()
     ASSERT_ERRNO(errno==ENOMEM, NULL);
 }
 
+void InvariantDataRealloc(bool aligned, size_t maxAllocSize, bool checkData)
+{
+    Harness::FastRandom fastRandom(1);
+    size_t size = 0, start = 0;
+    char *ptr = NULL,
+        // master to create copies and compare ralloc result against it
+        *master = (char*)Tmalloc(2*maxAllocSize);
+
+    ASSERT(master, NULL);
+    ASSERT(!(2*maxAllocSize%sizeof(unsigned short)),
+           "The loop below expects that 2*maxAllocSize contains sizeof(unsigned short)");
+    for (size_t k = 0; k<2*maxAllocSize; k+=sizeof(unsigned short))
+        *(unsigned short*)(master+k) = fastRandom.get();
+
+    for (int i=0; i<100; i++) {
+        // don't want sizeNew==0 here
+        const size_t sizeNew = fastRandom.get() % (maxAllocSize-1) + 1;
+        char *ptrNew = aligned?
+            (char*)Taligned_realloc(ptr, sizeNew, choose_random_alignment())
+            : (char*)Trealloc(ptr, sizeNew);
+        ASSERT(ptrNew, NULL);
+        // check that old data not changed
+        if (checkData)
+            ASSERT(!memcmp(ptrNew, master+start, min(size, sizeNew)), "broken data");
+
+        // prepare fresh data, copying them from random position in master
+        size = sizeNew;
+        ptr = ptrNew;
+        if (checkData) {
+            start = fastRandom.get() % maxAllocSize;
+            memcpy(ptr, master+start, size);
+        }
+    }
+    if (aligned)
+        Taligned_realloc(ptr, 0, choose_random_alignment());
+    else
+        Trealloc(ptr, 0);
+    Tfree(master);
+}
+
+#include "harness_memory.h"
+
+void CheckReallocLeak()
+{
+    int i;
+    const int ITER_TO_STABILITY = 10;
+    // do bootstrap
+    for (int k=0; k<3; k++)
+        InvariantDataRealloc(/*aligned=*/false, 128*MByte, /*checkData=*/false);
+    size_t prev = GetMemoryUsage(peakUsage);
+    // expect realloc to not increase peak memory consumption after ITER_TO_STABILITY-1 iterations
+    for (i=0; i<ITER_TO_STABILITY; i++) {
+        for (int k=0; k<3; k++)
+            InvariantDataRealloc(/*aligned=*/false, 128*MByte, /*checkData=*/false);
+        size_t curr = GetMemoryUsage(peakUsage);
+        if (prev == curr)
+            break;
+        prev = curr;
+    }
+    ASSERT(i < ITER_TO_STABILITY, "Can't stabilize memory consumption.");
+}
+
 HARNESS_EXPORT
 int main(int argc, char* argv[]) {
     argC=argc;
@@ -438,6 +502,7 @@ int main(int argc, char* argv[]) {
 #endif // _MSC_VER
 
     CheckArgumentsOverflow();
+    CheckReallocLeak();
     for( int p=MaxThread; p>=MinThread; --p ) {
         REMARK("testing with %d threads\n", p );
         for (int limit=0; limit<2; limit++) {
@@ -513,56 +578,6 @@ void* Taligned_realloc(void* memblock, size_t size, size_t alignment)
     return ret;
 }
 
-inline size_t choose_random_alignment() {
-    return sizeof(void*)<<(rand() % POWERS_OF_2);
-}
-
-void CMemTest::InvariantDataRealloc(bool aligned)
-{
-    size_t size, sizeMin;
-    CountErrors=0;
-    if (FullLog) REPORT("\nInvariant data by realloc....");
-    UCHAR* pchar;
-    sizeMin=size=rand()%MAX_SIZE+10;
-    pchar = aligned?
-        (UCHAR*)Taligned_realloc(NULL,size,choose_random_alignment())
-        : (UCHAR*)Trealloc(NULL,size);
-    if (NULL == pchar)
-        return;
-    for (size_t k=0; k<size; k++)
-        pchar[k]=(UCHAR)k%255+1;
-    for (int i=0; i<COUNTEXPERIMENT; i++)
-    {
-        size=rand()%MAX_SIZE+10;
-        UCHAR *pcharNew = aligned?
-            (UCHAR*)Taligned_realloc(pchar,size, choose_random_alignment())
-            : (UCHAR*)Trealloc(pchar,size);
-        if (NULL == pcharNew)
-            continue;
-        pchar = pcharNew;
-        sizeMin=size<sizeMin ? size : sizeMin;
-        for (size_t k=0; k<sizeMin; k++)
-            if (pchar[k] != (UCHAR)k%255+1)
-            {
-                CountErrors++;
-                if (ShouldReportError())
-                {
-                    REPORT("stand '%c', must stand '%c'\n",pchar[k],(UCHAR)k%255+1);
-                    REPORT("error: data changed (at %llu, SizeMin=%llu)\n",
-                           (long long unsigned)k,(long long unsigned)sizeMin);
-                }
-            }
-    }
-    if (aligned)
-        Taligned_realloc(pchar,0,choose_random_alignment());
-    else
-        Trealloc(pchar,0);
-    if (CountErrors) REPORT("%s\n",strError);
-    else if (FullLog) REPORT("%s\n",strOk);
-    error_occurred |= ( CountErrors>0 ) ;
-    //REPORT("end check\n");
-}
-
 struct PtrSize {
     void  *ptr;
     size_t size;
@@ -605,7 +620,7 @@ void CMemTest::AddrArifm()
         if (NULL!=tmpAddr) {
             arr[i].ptr = tmpAddr;
             arr[i].size = count;
-        } else if (count==0) { // becasue realloc(..., 0) works as free
+        } else if (count==0) { // because realloc(..., 0) works as free
             arr[i].ptr = NULL;
             arr[i].size = 0;
         }
@@ -717,7 +732,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
 
     /* There is a bug in the specific version of GLIBC (2.5-12) shipped
        with RHEL5 that leads to erroneous working of the test
-       on Intel64 and IPF systems when setrlimit-related part is enabled.
+       on Intel(R) 64 and Itanium(R) architecture when setrlimit-related part is enabled.
        Switching to GLIBC 2.5-18 from RHEL5.1 resolved the issue.
      */
     if (perProcessLimits)
@@ -774,7 +789,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                 // Technically, if malloc returns a non-NULL pointer, it is allowed to set errno anyway.
                 // However, on most systems it does not set errno.
                 bool known_issue = false;
-#if __linux__
+#if __linux__ || __ANDROID__
                 if( CHECK_ERRNO(errno==ENOMEM) ) known_issue = true;
 #endif /* __linux__ */
                 if ( CHECK_ERRNO(errno != ENOMEM+j+1) && !known_issue) {
@@ -836,19 +851,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
             {
                 errno = 0;
                 tmp=Trealloc(PointerList[i].Pointer,PointerList[i].Size*2);
-                if (PointerList[i].Pointer == tmp) // the same place
-                {
-                    bool known_issue = false;
-#if __linux__
-                    if( errno==ENOMEM ) known_issue = true;
-#endif /* __linux__ */
-                    if (errno != 0 && !known_issue) {
-                        CountErrors++;
-                        if (ShouldReportError()) REPORT("valid pointer returned, error: errno not kept\n");
-                    }
-                    PointerList[i].Size *= 2;
-                }
-                else if (tmp != PointerList[i].Pointer && tmp != NULL) // another place
+                if (tmp != NULL) // same or another place
                 {
                     bool known_issue = false;
 #if __linux__
@@ -862,9 +865,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
                     myMemset((char*)tmp + PointerList[i].Size, 0, PointerList[i].Size);
                     PointerList[i].Pointer = tmp;
                     PointerList[i].Size *= 2;
-                }
-                else if (tmp == NULL)
-                {
+                } else {
                     CountNULL++;
                     if ( CHECK_ERRNO(errno != ENOMEM) )
                     {
@@ -892,7 +893,7 @@ void CMemTest::NULLReturn(UINT MinSize, UINT MaxSize, int total_threads)
     else
         limitMem(0);
 }
-#endif /* #if __APPLE__ */
+#endif /* #if !__APPLE__ */
 
 void CMemTest::UniquePointer()
 {
@@ -1083,9 +1084,9 @@ void CMemTest::RunAllTests(int total_threads)
 {
     Zerofilling();
     Free_NULL();
-    InvariantDataRealloc(/*aligned=*/false);
+    InvariantDataRealloc(/*aligned=*/false, 8*MByte, /*checkData=*/true);
     if (Raligned_realloc)
-        InvariantDataRealloc(/*aligned=*/true);
+        InvariantDataRealloc(/*aligned=*/true, 8*MByte, /*checkData=*/true);
     TestAlignedParameters();
     UniquePointer();
     AddrArifm();
