@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_graph.h"
@@ -32,11 +24,12 @@
 
 #define N 300
 #define T 4 
-#define M 4
+#define M 5
 
 template< typename R >
 void simple_read_write_tests() {
-    tbb::flow::overwrite_node<R> n;
+    tbb::flow::graph g;
+    tbb::flow::overwrite_node<R> n(g);
 
     for ( int t = 0; t < T; ++t ) {
         R v0(N+1);
@@ -55,6 +48,14 @@ void simple_read_write_tests() {
            tbb::flow::make_edge( n, r[i] );
         }
 
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+        ASSERT(n.successor_count() == M, NULL);
+        typename tbb::flow::overwrite_node<R>::successor_list_type my_succs;
+        n.copy_successors(my_succs);
+        ASSERT(my_succs.size() == M, NULL);
+        ASSERT(n.predecessor_count() == 0, NULL);
+#endif
+
         for (int i = 0; i < N; ++i ) {
             R v1(static_cast<R>(i));
             ASSERT( n.try_put( v1 ), NULL );
@@ -100,7 +101,8 @@ public:
 
 template< typename R >
 void parallel_read_write_tests() {
-    tbb::flow::overwrite_node<R> n;
+    tbb::flow::graph g;
+    tbb::flow::overwrite_node<R> n(g);
     //Create a vector of identical nodes
     std::vector< tbb::flow::overwrite_node<R> > ow_vec(2, n);
 
@@ -148,6 +150,10 @@ int TestMain() {
         parallel_read_write_tests<int>();
         parallel_read_write_tests<float>();
     }
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    test_extract_on_node<tbb::flow::overwrite_node, int>();
+    test_extract_on_node<tbb::flow::overwrite_node, float>();
+#endif
     return Harness::Done;
 }
 
