@@ -1,39 +1,33 @@
 /*
-    Copyright 2005-2011 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 // Test for function template parallel_for.h
 
-// Enable testing of serial subset.
+// These features are pure additions and thus can be always "on" in the test
 #define TBB_PREVIEW_SERIAL_SUBSET 1
+#define TBB_PREVIEW_STATIC_PARTITIONER 1
+#include "harness_defs.h"
 
 #if _MSC_VER
 #pragma warning (push)
-#if !defined(__INTEL_COMPILER)
+#if __TBB_MSVC_UNREACHABLE_CODE_IGNORED
     // Suppress pointless "unreachable code" warning.
     #pragma warning (disable: 4702)
 #endif
@@ -45,6 +39,7 @@
 #define _SCL_SECURE_NO_WARNINGS
 #endif //#if _MSC_VER
 
+#include "harness_defs.h"
 #include "tbb/parallel_for.h"
 #include "tbb/atomic.h"
 #include "harness_assert.h"
@@ -115,65 +110,85 @@ static tbb::atomic<int> Array[N];
 
 struct serial_tag {};
 struct parallel_tag {};
+struct empty_partitioner_tag {};
 
-template <typename Flavor, typename Range, typename Body>
+template <typename Flavor, typename Partitioner, typename Range, typename Body>
 struct Invoker;
 
+#if TBB_PREVIEW_SERIAL_SUBSET
 template <typename Range, typename Body>
-struct Invoker<serial_tag, Range, Body> {
-    void operator()( const Range& r, const Body& body, const tbb::simple_partitioner& p ) {
+struct Invoker<serial_tag, empty_partitioner_tag, Range, Body> {
+    void operator()( const Range& r, const Body& body, empty_partitioner_tag& ) {
+        tbb::serial:: parallel_for( r, body );
+    }
+};
+template <typename Partitioner, typename Range, typename Body>
+struct Invoker<serial_tag, Partitioner, Range, Body> {
+    void operator()( const Range& r, const Body& body, Partitioner& p ) {
         tbb::serial:: parallel_for( r, body, p );
     }
-    void operator()( const Range& r, const Body& body, const tbb::auto_partitioner& p ) {
-        tbb::serial:: parallel_for( r, body, p );
-    }
-    void operator()( const Range& r, const Body& body, tbb::affinity_partitioner& p ) {
-        tbb::serial:: parallel_for( r, body, p );
-    }
-    void operator()( const Range& r, const Body& body ) {
-        tbb::serial:: parallel_for( r, body, tbb::auto_partitioner() );
+};
+#endif
+
+template <typename Range, typename Body>
+struct Invoker<parallel_tag, empty_partitioner_tag, Range, Body> {
+    void operator()( const Range& r, const Body& body, empty_partitioner_tag& ) {
+        tbb:: parallel_for( r, body );
     }
 };
 
-template <typename Range, typename Body>
-struct Invoker<parallel_tag, Range, Body> {
-    void operator()( const Range& r, const Body& body, const tbb::simple_partitioner& p ) {
+template <typename Partitioner, typename Range, typename Body>
+struct Invoker<parallel_tag, Partitioner, Range, Body> {
+    void operator()( const Range& r, const Body& body, Partitioner& p ) {
         tbb:: parallel_for( r, body, p );
     }
-    void operator()( const Range& r, const Body& body, const tbb::auto_partitioner& p ) {
-        tbb:: parallel_for( r, body, p );
-    }
-    void operator()( const Range& r, const Body& body, tbb::affinity_partitioner& p ) {
-        tbb:: parallel_for( r, body, p );
-    }
-    void operator()( const Range& r, const Body& body ) {
-        tbb:: parallel_for( r, body, tbb::auto_partitioner() );
-    }
 };
 
-template <typename Flavor, typename T, typename Body>
+template <typename Flavor, typename Partitioner, typename T, typename Body>
 struct InvokerStep;
 
+#if TBB_PREVIEW_SERIAL_SUBSET
 template <typename T, typename Body>
-struct InvokerStep<serial_tag, T, Body> {
-    void operator()( const T& first, const T& last, const Body& f ) {
+struct InvokerStep<serial_tag, empty_partitioner_tag, T, Body> {
+    void operator()( const T& first, const T& last, const Body& f, empty_partitioner_tag& ) {
         tbb::serial:: parallel_for( first, last, f );
     }
-    void operator()( const T& first, const T& last, const T& step, const Body& f ) {
+    void operator()( const T& first, const T& last, const T& step, const Body& f, empty_partitioner_tag& ) {
         tbb::serial:: parallel_for( first, last, step, f );
     }
 };
 
+template <typename Partitioner, typename T, typename Body>
+struct InvokerStep<serial_tag, Partitioner, T, Body> {
+    void operator()( const T& first, const T& last, const Body& f, Partitioner& p ) {
+        tbb::serial:: parallel_for( first, last, f, p);
+    }
+    void operator()( const T& first, const T& last, const T& step, const Body& f, Partitioner& p ) {
+        tbb::serial:: parallel_for( first, last, step, f, p );
+    }
+};
+#endif
+
 template <typename T, typename Body>
-struct InvokerStep<parallel_tag, T, Body> {
-    void operator()( const T& first, const T& last, const Body& f ) {
+struct InvokerStep<parallel_tag, empty_partitioner_tag, T, Body> {
+    void operator()( const T& first, const T& last, const Body& f, empty_partitioner_tag& ) {
         tbb:: parallel_for( first, last, f );
     }
-    void operator()( const T& first, const T& last, const T& step, const Body& f ) {
+    void operator()( const T& first, const T& last, const T& step, const Body& f, empty_partitioner_tag& ) {
         tbb:: parallel_for( first, last, step, f );
     }
 };
 
+template <typename Partitioner, typename T, typename Body>
+struct InvokerStep<parallel_tag, Partitioner, T, Body> {
+    void operator()( const T& first, const T& last, const Body& f, Partitioner& p ) {
+        tbb:: parallel_for( first, last, f, p );
+    }
+    void operator()( const T& first, const T& last, const T& step, const Body& f, Partitioner& p ) {
+        tbb:: parallel_for( first, last, step, f, p );
+    }
+};
+
 template<typename Flavor, size_t Pad>
 void Flog( int nthread ) {
     tbb::tick_count T0 = tbb::tick_count::now();
@@ -185,21 +200,28 @@ void Flog( int nthread ) {
             const FooBody<Pad> fc = f;
             memset( Array, 0, sizeof(Array) );
             FooBodyCount = 1;
-            Invoker< Flavor, FooRange<Pad>, FooBody<Pad> > invoke_for;
             switch (mode) {
-                case 0:
-                    invoke_for( rc, fc );
+            case 0: {
+                empty_partitioner_tag p;
+                Invoker< Flavor, empty_partitioner_tag, FooRange<Pad>, FooBody<Pad> > invoke_for;
+                invoke_for( rc, fc, p );
+            }
                 break;
-                case 1:
-                    invoke_for( rc, fc, tbb::simple_partitioner() );
+            case 1: {
+                Invoker< Flavor, const tbb::simple_partitioner, FooRange<Pad>, FooBody<Pad> > invoke_for;
+                invoke_for( rc, fc, tbb::simple_partitioner() );
+            }
                 break;
-                case 2:
-                    invoke_for( rc, fc, tbb::auto_partitioner() );
+            case 2: {
+                Invoker< Flavor, const tbb::auto_partitioner, FooRange<Pad>, FooBody<Pad> > invoke_for;
+                invoke_for( rc, fc, tbb::auto_partitioner() );
+            }
                 break;
-                case 3: {
-                    static tbb::affinity_partitioner affinity;
-                    invoke_for( rc, fc, affinity );
-                }
+            case 3: {
+                static tbb::affinity_partitioner affinity;
+                Invoker< Flavor, tbb::affinity_partitioner, FooRange<Pad>, FooBody<Pad> > invoke_for;
+                invoke_for( rc, fc, affinity );
+            }
                 break;
             }
             for( int j=0; j<i; ++j )
@@ -239,21 +261,21 @@ public:
     #pragma warning (pop)
 #endif
 
-template <typename Flavor, typename T>
-void TestParallelForWithStepSupport()
+template <typename Flavor, typename T, typename Partitioner>
+void TestParallelForWithStepSupportHelper(Partitioner& p)
 {
     const T pfor_buffer_test_size = static_cast<T>(PFOR_BUFFER_TEST_SIZE);
     const T pfor_buffer_actual_size = static_cast<T>(PFOR_BUFFER_ACTUAL_SIZE);
     // Testing parallel_for with different step values
-    InvokerStep< Flavor, T, TestFunctor<T> > invoke_for;
+    InvokerStep< Flavor, Partitioner, T, TestFunctor<T> > invoke_for;
     for (T begin = 0; begin < pfor_buffer_test_size - 1; begin += pfor_buffer_test_size / 10 + 1) {
         T step;
         for (step = 1; step < pfor_buffer_test_size; step++) {
             memset(pfor_buffer, 0, pfor_buffer_actual_size * sizeof(size_t));
             if (step == 1){
-                invoke_for(begin, pfor_buffer_test_size, TestFunctor<T>());
+                invoke_for(begin, pfor_buffer_test_size, TestFunctor<T>(), p);
             } else {
-                invoke_for(begin, pfor_buffer_test_size, step, TestFunctor<T>());
+                invoke_for(begin, pfor_buffer_test_size, step, TestFunctor<T>(), p);
             }
             // Verifying that parallel_for processed all items it should
             for (T i = begin; i < pfor_buffer_test_size; i = i + step) {
@@ -266,6 +288,21 @@ void TestParallelForWithStepSupport()
             }
         }
     }
+}
+
+template <typename Flavor, typename T>
+void TestParallelForWithStepSupport()
+{
+    static tbb::affinity_partitioner affinity_p;
+    tbb::auto_partitioner auto_p;
+    tbb::simple_partitioner simple_p;
+    empty_partitioner_tag p;
+
+    // Try out all partitioner combinations
+    TestParallelForWithStepSupportHelper< Flavor,T,empty_partitioner_tag >(p);
+    TestParallelForWithStepSupportHelper< Flavor,T,const tbb::auto_partitioner >(auto_p);
+    TestParallelForWithStepSupportHelper< Flavor,T,const tbb::simple_partitioner >(simple_p);
+    TestParallelForWithStepSupportHelper< Flavor,T,tbb::affinity_partitioner >(affinity_p);
 
     // Testing some corner cases
     tbb::parallel_for(static_cast<T>(2), static_cast<T>(1), static_cast<T>(1), TestFunctor<T>());
@@ -411,6 +448,142 @@ void TestSimplePartitionerStability(){
 #include <cstdio>
 #include "tbb/task_scheduler_init.h"
 #include "harness_cpu.h"
+#include "harness_barrier.h"
+#include "test_partitioner.h"
+
+namespace interaction_with_range_and_partitioner {
+
+// Test checks compatibility of parallel_for algorithm with various range implementations
+
+void test() {
+    using namespace test_partitioner_utils::interaction_with_range_and_partitioner;
+
+    test_partitioner_utils::SimpleBody b;
+    tbb::affinity_partitioner ap;
+
+    parallel_for(Range1(true, false), b, ap);
+    parallel_for(Range2(true, false), b, ap);
+    parallel_for(Range3(true, false), b, ap);
+    parallel_for(Range4(false, true), b, ap);
+    parallel_for(Range5(false, true), b, ap);
+    parallel_for(Range6(false, true), b, ap);
+
+    parallel_for(Range1(false, true), b, tbb::simple_partitioner());
+    parallel_for(Range2(false, true), b, tbb::simple_partitioner());
+    parallel_for(Range3(false, true), b, tbb::simple_partitioner());
+    parallel_for(Range4(false, true), b, tbb::simple_partitioner());
+    parallel_for(Range5(false, true), b, tbb::simple_partitioner());
+    parallel_for(Range6(false, true), b, tbb::simple_partitioner());
+
+    parallel_for(Range1(false, true), b, tbb::auto_partitioner());
+    parallel_for(Range2(false, true), b, tbb::auto_partitioner());
+    parallel_for(Range3(false, true), b, tbb::auto_partitioner());
+    parallel_for(Range4(false, true), b, tbb::auto_partitioner());
+    parallel_for(Range5(false, true), b, tbb::auto_partitioner());
+    parallel_for(Range6(false, true), b, tbb::auto_partitioner());
+}
+
+} // namespace interaction_with_range_and_partitioner
+
+namespace various_range_implementations {
+
+using namespace test_partitioner_utils;
+using namespace test_partitioner_utils::TestRanges;
+
+// Body ensures that initial work distribution is done uniformly through affinity mechanism and not
+// through work stealing
+class Body {
+    Harness::SpinBarrier &m_sb;
+public:
+    Body(Harness::SpinBarrier& sb) : m_sb(sb) { }
+    Body(Body& b, tbb::split) : m_sb(b.m_sb) { }
+    Body& operator =(const Body&) { return *this; }
+    template <typename Range>
+    void operator()(Range& r) const {
+        REMARK("Executing range [%lu, %lu)\n", r.begin(), r.end());
+        m_sb.timed_wait(10); // waiting for all threads
+    }
+};
+
+namespace correctness {
+
+/* Testing only correctness (that is parallel_for does not hang) */
+template <typename RangeType, bool /* feedback */, bool ensure_non_emptiness>
+void test() {
+    static const int thread_num = tbb::task_scheduler_init::default_num_threads();
+    RangeType range( 0, thread_num, NULL, false, ensure_non_emptiness );
+    tbb::affinity_partitioner ap;
+    tbb::parallel_for( range, SimpleBody(), ap );
+}
+
+} // namespace correctness
+
+namespace uniform_distribution {
+
+/* Body of parallel_for algorithm would hang if non-uniform work distribution happened  */
+template <typename RangeType, bool feedback, bool ensure_non_emptiness>
+void test() {
+    static const int thread_num = tbb::task_scheduler_init::default_num_threads();
+    Harness::SpinBarrier sb( thread_num );
+    RangeType range(0, thread_num, NULL, feedback, ensure_non_emptiness);
+    const Body sync_body( sb );
+    tbb::affinity_partitioner ap;
+    tbb::parallel_for( range, sync_body, ap );
+#if TBB_PREVIEW_STATIC_PARTITIONER
+    tbb::parallel_for( range, sync_body, tbb::static_partitioner() );
+#endif
+}
+
+} // namespace uniform_distribution
+
+void test() {
+    const bool provide_feedback = __TBB_ENABLE_RANGE_FEEDBACK;
+    const bool ensure_non_empty_range = true;
+
+    // BlockedRange does not take into account feedback and non-emptiness settings but uses the
+    // tbb::blocked_range implementation
+    uniform_distribution::test<BlockedRange, !provide_feedback, !ensure_non_empty_range>();
+
+#if __TBB_ENABLE_RANGE_FEEDBACK
+    using uniform_distribution::test; // if feedback is enabled ensure uniform work distribution
+#else
+    using correctness::test;
+#endif
+
+    {
+        test<RoundedDownRange, provide_feedback, ensure_non_empty_range>();
+        test<RoundedDownRange, provide_feedback, !ensure_non_empty_range>();
+#if __TBB_ENABLE_RANGE_FEEDBACK && !__TBB_MIC_NATIVE
+        // due to fast division algorithm on MIC
+        test<RoundedDownRange, !provide_feedback, ensure_non_empty_range>();
+        test<RoundedDownRange, !provide_feedback, !ensure_non_empty_range>();
+#endif
+    }
+
+    {
+        test<RoundedUpRange, provide_feedback, ensure_non_empty_range>();
+        test<RoundedUpRange, provide_feedback, !ensure_non_empty_range>();
+#if __TBB_ENABLE_RANGE_FEEDBACK && !__TBB_MIC_NATIVE
+        // due to fast division algorithm on MIC
+        test<RoundedUpRange, !provide_feedback, ensure_non_empty_range>();
+        test<RoundedUpRange, !provide_feedback, !ensure_non_empty_range>();
+#endif
+    }
+
+    // Testing that parallel_for algorithm works with such weird ranges
+    correctness::test<Range1_2, /* provide_feedback= */ false, !ensure_non_empty_range>();
+    correctness::test<Range1_999, /* provide_feedback= */ false, !ensure_non_empty_range>();
+    correctness::test<Range999_1, /* provide_feedback= */ false, !ensure_non_empty_range>();
+
+    // The following ranges do not comply with the proportion suggested by partitioner. Therefore
+    // they have to provide the proportion in which they were actually split back to partitioner and
+    // ensure theirs non-emptiness
+    test<Range1_2, provide_feedback, ensure_non_empty_range>();
+    test<Range1_999, provide_feedback, ensure_non_empty_range>();
+    test<Range999_1, provide_feedback, ensure_non_empty_range>();
+}
+
+} // namespace various_range_implementations
 
 int TestMain () {
     if( MinThread<1 ) {
@@ -437,6 +610,7 @@ int TestMain () {
             TestParallelForWithStepSupport<parallel_tag,unsigned long long>();
             TestParallelForWithStepSupport<parallel_tag,size_t>();
 
+#if TBB_PREVIEW_SERIAL_SUBSET
             // This is for testing serial implementation.
             if( p == MaxThread ) {
                 Flog<serial_tag,1>(p);
@@ -452,6 +626,7 @@ int TestMain () {
                 TestParallelForWithStepSupport<serial_tag,unsigned long long>();
                 TestParallelForWithStepSupport<serial_tag,size_t>();
             }
+#endif
 
 #if TBB_USE_EXCEPTIONS && !__TBB_THROW_ACROSS_MODULE_BOUNDARY_BROKEN
             TestExceptionsSupport();
@@ -477,8 +652,11 @@ int TestMain () {
     REPORT("Known issue: exception handling tests are skipped.\n");
 #endif
 #if (HAVE_m128 || HAVE_m256) && __TBB_SSE_STACK_ALIGNMENT_BROKEN
-    REPORT("Known issue: stack alignment for SSE/AVX not tested.\n");
+    REPORT("Known issue: stack alignment for SIMD instructions not tested.\n");
 #endif
+
+    various_range_implementations::test();
+    interaction_with_range_and_partitioner::test();
     return Harness::Done;
 }
 
