@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 // Before including pipeline.h, set up the variable to count heap allocated
@@ -46,7 +38,6 @@ const unsigned n_buffers = 2*n_tokens;
 const unsigned max_counter = 16;
 static tbb::atomic<int> output_counter;
 static tbb::atomic<int> input_counter;
-static tbb::atomic<int> check_type_counter;
 static tbb::atomic<int> non_pointer_specialized_calls;
 static tbb::atomic<int> pointer_specialized_calls;
 static tbb::atomic<int> first_pointer_specialized_calls;
@@ -89,39 +80,7 @@ private:
     T *my_p;
 };
 
-template<class Counter>
-class check_type : Harness::NoAfterlife {
-    Counter id;
-    bool am_ready;
-public:
-    check_type( ) : id(0), am_ready(false) {
-        ++check_type_counter;
-    }
-
-    check_type(const check_type& other) : Harness::NoAfterlife(other) {
-        other.AssertLive();
-        AssertLive();
-        id = other.id;
-        am_ready = other.am_ready;
-        ++check_type_counter;
-    }
-
-    ~check_type() { 
-        AssertLive(); 
-        --check_type_counter;
-        ASSERT(check_type_counter >= 0, "too many destructions");
-    }
-
-    unsigned int my_id() { AssertLive(); return id; }
-    bool is_ready() { AssertLive(); return am_ready; }
-    void function() {
-        AssertLive();
-        if( id == 0 ) {
-            id = 1;
-            am_ready = true;
-        }
-    }
-};
+#include "harness_checktype.h"
 
 // methods for testing check_type< >, that return okay values for other types.
 template<typename T>
@@ -400,10 +359,6 @@ void checkCounters(final_assert_type my_t) {
         case no_pointer_counts:
             break;
     }
-    if(check_type_counter > 0) {
-        REMARK("check_type_counter == %lu\n", (unsigned long)check_type_counter);
-    }
-    ASSERT(!check_type_counter, "Error in check_type creation/destruction");
 }
 
 static const tbb::filter::mode filter_table[] = { tbb::filter::parallel, tbb::filter::serial_in_order, tbb::filter::serial_out_of_order}; 
@@ -626,6 +581,9 @@ void run_function(const char *l1, const char *l2) {
     if(check_intbuffer) REMARK(", check output of filters");
     REMARK("\n");
 
+    Check<type1> check1;  // check constructions/destructions
+    Check<type2> check2;  // for type1 or type2 === check_type<T>
+
     const size_t number_of_filters = 3;
 
     input_filter<type1> i_filter;
@@ -698,18 +656,12 @@ int TestMain() {
         run_function<size_t,double>("size_t", "double");
         run_function<size_t,bool>("size_t", "bool");
         run_function<int,int>("int","int");
-        check_type_counter = 0;
         run_function<check_type<unsigned int>,size_t>("check_type<unsigned int>", "size_t");
-        ASSERT(!check_type_counter, "Error in check_type<unsigned int> creation/destruction");
         run_function<check_type<unsigned short>,size_t>("check_type<unsigned short>", "size_t");
-        ASSERT(!check_type_counter, "Error in check_type<unsigned short> creation/destruction");
         run_function<check_type<unsigned int>, check_type<unsigned int> >("check_type<unsigned int>", "check_type<unsigned int>");
         run_function<check_type<unsigned int>, check_type<unsigned short> >("check_type<unsigned int>", "check_type<unsigned short>");
-        ASSERT(!check_type_counter, "Error in check_type<unsigned int> creation/destruction");
         run_function<check_type<unsigned short>, check_type<unsigned short> >("check_type<unsigned short>", "check_type<unsigned short>");
-        ASSERT(!check_type_counter, "Error in check_type<unsigned short> creation/destruction");
         run_function<double, check_type<unsigned short> >("double", "check_type<unsigned short>");
-        ASSERT(!check_type_counter, "Error in check_type<unsigned short> creation/destruction");
     }
     return Harness::Done;
 }
