@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "tbb/parallel_reduce.h"
@@ -268,6 +260,42 @@ public:
 
 #include "tbb/task_scheduler_init.h"
 #include "harness_cpu.h"
+#include "test_partitioner.h"
+
+namespace interaction_with_range_and_partitioner {
+
+// Test checks compatibility of parallel_reduce algorithm with various range implementations
+
+void test() {
+    using namespace test_partitioner_utils::interaction_with_range_and_partitioner;
+
+    test_partitioner_utils::SimpleReduceBody body;
+    tbb::affinity_partitioner ap;
+
+    parallel_reduce(Range1(true, false), body, ap);
+    parallel_reduce(Range2(true, false), body, ap);
+    parallel_reduce(Range3(true, false), body, ap);
+    parallel_reduce(Range4(false, true), body, ap);
+    parallel_reduce(Range5(false, true), body, ap);
+    parallel_reduce(Range6(false, true), body, ap);
+
+    parallel_reduce(Range1(false, true), body, tbb::simple_partitioner());
+    parallel_reduce(Range2(false, true), body, tbb::simple_partitioner());
+    parallel_reduce(Range3(false, true), body, tbb::simple_partitioner());
+    parallel_reduce(Range4(false, true), body, tbb::simple_partitioner());
+    parallel_reduce(Range5(false, true), body, tbb::simple_partitioner());
+    parallel_reduce(Range6(false, true), body, tbb::simple_partitioner());
+
+    parallel_reduce(Range1(false, true), body, tbb::auto_partitioner());
+    parallel_reduce(Range2(false, true), body, tbb::auto_partitioner());
+    parallel_reduce(Range3(false, true), body, tbb::auto_partitioner());
+    parallel_reduce(Range4(false, true), body, tbb::auto_partitioner());
+    parallel_reduce(Range5(false, true), body, tbb::auto_partitioner());
+    parallel_reduce(Range6(false, true), body, tbb::auto_partitioner());
+}
+
+} // interaction_with_range_and_partitioner
+
 
 int TestMain () {
     if( MinThread<0 ) {
@@ -283,5 +311,7 @@ int TestMain () {
         // Test that all workers sleep when no work
         TestCPUUserTime(p);
     }
+
+    interaction_with_range_and_partitioner::test();
     return Harness::Done;
 }
