@@ -1,5 +1,5 @@
 /*
-    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
     This file is part of Threading Building Blocks. Threading Building Blocks is free software;
     you can redistribute it and/or modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
     reasons why the executable file might be covered by the GNU General Public License.
 */
 
+// have to expose the reset_node method to be able to reset a function_body
 #include "harness.h"
 #include "harness_graph.h"
 #include "tbb/flow_graph.h"
@@ -33,20 +34,23 @@ class test_push_receiver : public tbb::flow::receiver<T> {
 
 public:
 
-    test_push_receiver() {  
-        for (int i = 0; i < N; ++i ) 
+    test_push_receiver() {
+        for (int i = 0; i < N; ++i )
             my_counters[i] = 0;
     }
 
     int get_count( int i ) {
-       int v = my_counters[i]; 
+       int v = my_counters[i];
        return v;
     }
 
-    typedef tbb::flow::sender<T> predecessor_type;
+    typedef typename tbb::flow::receiver<T>::predecessor_type predecessor_type;
 
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    typedef typename tbb::flow::receiver<T>::built_predecessors_type built_predecessors_type;
     typedef typename tbb::flow::receiver<T>::predecessor_list_type predecessor_list_type;
+    built_predecessors_type bpt;
+    built_predecessors_type &built_predecessors() { return bpt; }
     void internal_add_built_predecessor( predecessor_type & ) { }
     void internal_delete_built_predecessor( predecessor_type & ) { }
     void copy_predecessors( predecessor_list_type & ) { }
@@ -56,16 +60,12 @@ public:
     tbb::task *try_put_task( const T &v ) {
        int i = (int)v;
        ++my_counters[i];
-       return const_cast<tbb::task *>(tbb::flow::interface7::SUCCESSFULLY_ENQUEUED);
+       return const_cast<tbb::task *>(tbb::flow::interface8::SUCCESSFULLY_ENQUEUED);
     }
 
 
 
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
     /*override*/void reset_receiver(tbb::flow::reset_flags /*f*/) {}
-#else
-    /*override*/void reset_receiver() {}
-#endif
 };
 
 template< typename T >
@@ -73,7 +73,7 @@ class source_body {
 
    tbb::atomic<int> my_count;
    int *ninvocations;
- 
+
 public:
 
    source_body() : ninvocations(NULL) { my_count = 0; }
@@ -86,7 +86,7 @@ public:
          return true;
       else
          return false;
-   } 
+   }
 
 };
 
@@ -97,15 +97,15 @@ class function_body {
 
 public:
 
-    function_body( tbb::atomic<int> *counters ) : my_counters(counters) {  
-        for (int i = 0; i < N; ++i ) 
+    function_body( tbb::atomic<int> *counters ) : my_counters(counters) {
+        for (int i = 0; i < N; ++i )
             my_counters[i] = 0;
     }
 
     bool operator()( T v ) {
         ++my_counters[(int)v];
         return true;
-    } 
+    }
 
 };
 
@@ -119,7 +119,7 @@ void test_single_dest() {
    tbb::flow::make_edge( src, dest );
    g.wait_for_all();
    for (int i = 0; i < N; ++i ) {
-       ASSERT( dest.get_count(i) == 1, NULL ); 
+       ASSERT( dest.get_count(i) == 1, NULL );
    }
 
    // push only
@@ -131,10 +131,10 @@ void test_single_dest() {
    g.wait_for_all();
    for (int i = 0; i < N; ++i ) {
        int v = counters3[i];
-       ASSERT( v == 1, NULL ); 
+       ASSERT( v == 1, NULL );
    }
 
-   // push & pull 
+   // push & pull
    tbb::flow::source_node<T> src2(g, source_body<T>() );
    tbb::atomic<int> counters2[N];
    function_body<T> b2( counters2 );
@@ -149,7 +149,7 @@ void test_single_dest() {
    g.wait_for_all();
    for (int i = 0; i < N; ++i ) {
        int v = counters2[i];
-       ASSERT( v == 1, NULL ); 
+       ASSERT( v == 1, NULL );
    }
 
    // test copy constructor
@@ -158,10 +158,99 @@ void test_single_dest() {
    ASSERT( src_copy.register_successor(dest_c), NULL );
    g.wait_for_all();
    for (int i = 0; i < N; ++i ) {
-       ASSERT( dest_c.get_count(i) == 1, NULL ); 
+       ASSERT( dest_c.get_count(i) == 1, NULL );
    }
 }
 
+void test_reset() {
+    //    source_node -> function_node
+    tbb::flow::graph g;
+    tbb::atomic<int> counters3[N];
+    tbb::flow::source_node<int> src3(g, source_body<int>() );
+    tbb::flow::source_node<int> src_inactive(g, source_body<int>(), /*active*/ false );
+    function_body<int> b3( counters3 );
+    tbb::flow::function_node<int,bool> dest3(g, tbb::flow::unlimited, b3 );
+    tbb::flow::make_edge( src3, dest3 );
+    //    source_node already in active state.  Let the graph run,
+    g.wait_for_all();
+    //    check the array for each value.
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 1, NULL );
+        counters3[i] = 0;
+    }
+    g.reset(tbb::flow::rf_reset_bodies);  // <-- re-initializes the counts.
+    // and spawns task to run source
+    g.wait_for_all();
+    //    check output queue again.  Should be the same contents.
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 1, NULL );
+        counters3[i] = 0;
+    }
+    g.reset();  // doesn't reset the source_node_body to initial state, but does spawn a task
+                // to run the source_node.
+
+    g.wait_for_all();
+    // array should be all zero
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 0, NULL );
+    }
+
+    remove_edge(src3, dest3);
+    make_edge(src_inactive, dest3);
+
+    // src_inactive doesn't run
+    g.wait_for_all();
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 0, NULL );
+    }
+
+    // run graph
+    src_inactive.activate();
+    g.wait_for_all();
+    // check output
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 1, NULL );
+        counters3[i] = 0;
+    }
+    g.reset(tbb::flow::rf_reset_bodies);  // <-- reinitializes the counts
+    // src_inactive doesn't run
+    g.wait_for_all();
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 0, NULL );
+    }
+
+    // start it up
+    src_inactive.activate();
+    g.wait_for_all();
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 1, NULL );
+        counters3[i] = 0;
+    }
+    g.reset();  // doesn't reset the source_node_body to initial state, and doesn't
+                // spawn a task to run the source_node.
+
+    g.wait_for_all();
+    // array should be all zero
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 0, NULL );
+    }
+    src_inactive.activate();
+    // source_node_body is already in final state, so source_node will not forward a message.
+    g.wait_for_all();
+    for (int i = 0; i < N; ++i ) {
+        int v = counters3[i];
+        ASSERT( v == 0, NULL );
+    }
+}
+
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
 void test_extract() {
     int counts = 0;
@@ -169,7 +258,7 @@ void test_extract() {
     tbb::flow::graph g;
     typedef tbb::flow::source_node<int> snode_type;
     typedef snode_type::successor_list_type successor_list_type;
-    snode_type s0(g, source_body<int>(counts), /*is_active*/false ); 
+    snode_type s0(g, source_body<int>(counts), /*is_active*/false );
     tbb::flow::join_node< tbb::flow::tuple<int,int>, tbb::flow::reserving > j0(g);
     tbb::flow::join_node< tbb::flow::tuple<int,int>, tbb::flow::reserving > j1(g);
     tbb::flow::join_node< tbb::flow::tuple<int,int>, tbb::flow::reserving > j2(g);
@@ -193,7 +282,7 @@ void test_extract() {
     s0.activate();
     g.wait_for_all();  // no successors, so the body will not execute
     ASSERT(counts == 0, "source_node shouldn't forward (no successors)");
-    s0.extract(tbb::flow::rf_reset_bodies);
+    g.reset(tbb::flow::rf_reset_bodies);
 
     tbb::flow::make_edge(s0, tbb::flow::get<0>(j0.input_ports()));
     tbb::flow::make_edge(s0, tbb::flow::get<0>(j1.input_ports()));
@@ -308,7 +397,7 @@ void test_extract() {
 }
 #endif  /* TBB_PREVIEW_FLOW_GRAPH_FEATURES */
 
-int TestMain() { 
+int TestMain() {
     if( MinThread<1 ) {
         REPORT("number of threads must be positive\n");
         exit(1);
@@ -318,6 +407,7 @@ int TestMain() {
         test_single_dest<int>();
         test_single_dest<float>();
     }
+    test_reset();
 #if TBB_PREVIEW_FLOW_GRAPH_FEATURES
     test_extract();
 #endif
