@@ -1,36 +1,29 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
-#if __TBB_CPF_BUILD
-    #define TBB_PREVIEW_TASK_ARENA 1
-#endif
+#include "tbb/tbb_config.h"
 #include "harness.h"
 
+#if __TBB_GCC_STRICT_ALIASING_BROKEN
+    #pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#endif
 
 #if __TBB_TASK_GROUP_CONTEXT
 
@@ -50,8 +43,10 @@ int MinBaseDepth = 8;
 int MaxBaseDepth = 10;
 int BaseDepth = 0;
 
+const int DesiredNumThreads = 12;
+
 const int NumTests = 8;
-const int TestRepeats = 4;
+int TestSwitchBetweenMastersRepeats = 4;
 
 int g_NumMasters = 0;
 volatile intptr_t *g_LeavesExecuted = NULL;
@@ -71,8 +66,8 @@ namespace tbb {
 }
 #endif /* __TBB_TASK_PRIORITY */
 
-tbb::priority_t Low = tbb::priority_normal,
-                High = tbb::priority_high;
+tbb::priority_t Low,
+                High;
 int PreemptionActivatorId = 1;
 
 enum Options {
@@ -156,7 +151,7 @@ protected:
         parent_node->set_ref_count(NumLeafTasks + 1);
         --m_depth;
         for ( int i = 0; i < NumLeafTasks; ++i ) {
-            task *t = m_depth ? (task*) new(parent_node->allocate_child()) NodeType(m_tid, m_depth, m_opts, m_root) 
+            task *t = m_depth ? (task*) new(parent_node->allocate_child()) NodeType(m_tid, m_depth, m_opts, m_root)
                               : (task*) new(parent_node->allocate_child()) LeafTask(m_tid, m_opts);
             task::spawn(*t);
         }
@@ -299,9 +294,14 @@ void RunPrioritySwitchBetweenTwoMasters ( int idx, uintptr_t opts ) {
 }
 
 void TestPrioritySwitchBetweenTwoMasters () {
+    if ( P > DesiredNumThreads ) {
+        REPORT_ONCE( "Known issue: TestPrioritySwitchBetweenTwoMasters is skipped for big number of threads\n" );
+        return;
+    }
+    tbb::task_scheduler_init init; // keeps the market alive to reduce the amount of TBB warnings
     REMARK( "Stress tests: %s / %s \n", Low == tbb::priority_low ? "Low" : "Normal", High == tbb::priority_normal ? "Normal" : "High" );
     PrepareGlobals( 2 );
-    for ( int i = 0; i < TestRepeats; ++i ) {
+    for ( int i = 0; i < TestSwitchBetweenMastersRepeats; ++i ) {
         for ( BaseDepth = MinBaseDepth; BaseDepth <= MaxBaseDepth; ++BaseDepth ) {
             RunPrioritySwitchBetweenTwoMasters<BlockingNodeTask>( 0, NoPriorities );
             RunPrioritySwitchBetweenTwoMasters<BlockingNodeTask>( 1, TestPreemption );
@@ -316,13 +316,13 @@ void TestPrioritySwitchBetweenTwoMasters () {
         }
     }
 #if __TBB_TASK_PRIORITY
-    const int NumRuns = TestRepeats * (MaxBaseDepth - MinBaseDepth + 1);
+    const int NumRuns = TestSwitchBetweenMastersRepeats * (MaxBaseDepth - MinBaseDepth + 1);
     for ( int i = 0; i < NumTests; ++i ) {
         if ( g_TestFailures[i] )
             REMARK( "Test %d: %d failures in %d runs\n", i, g_TestFailures[i], NumRuns );
         if ( g_TestFailures[i] * 100 / NumRuns > 50 ) {
             if ( i == 1 )
-                REPORT( "Known issue: priority effect is limited in case of blocking-style nesting\n" );
+                REPORT_ONCE( "Known issue: priority effect is limited in case of blocking-style nesting\n" );
             else
                 REPORT( "Warning: test %d misbehaved too often (%d out of %d)\n", i, g_TestFailures[i], NumRuns );
         }
@@ -371,6 +371,9 @@ void EmulateWork( int ) {
 
 class PeriodicActivitiesBody {
 public:
+    static const int parallelIters[2];
+    static const int seqIters[2];
+    static int mode;
     void operator() ( int id ) const {
         tbb::task_group_context ctx;
 #if __TBB_TASK_PRIORITY
@@ -378,13 +381,17 @@ public:
 #else /* !__TBB_TASK_PRIORITY */
         (void)id;
 #endif /* !__TBB_TASK_PRIORITY */
-        for ( int i = 0; i < 5; ++i ) {
+        for ( int i = 0; i < seqIters[mode]; ++i ) {
             tbb::task_scheduler_init init;
-            tbb::parallel_for( 1, 10000, &EmulateWork, ctx );
+            tbb::parallel_for( 1, parallelIters[mode], &EmulateWork, ctx );
         }
     }
 };
 
+const int PeriodicActivitiesBody::parallelIters[] = {10000, 100};
+const int PeriodicActivitiesBody::seqIters[] = {5, 2};
+int PeriodicActivitiesBody::mode = 0;
+
 void TestPeriodicConcurrentActivities () {
     REMARK( "TestPeriodicConcurrentActivities: %s / %s \n", Low == tbb::priority_low ? "Low" : "Normal", High == tbb::priority_normal ? "Normal" : "High" );
     NativeParallelFor ( 2, PeriodicActivitiesBody() );
@@ -395,6 +402,7 @@ void TestPeriodicConcurrentActivities () {
 void TestPriorityAssertions () {
 #if TRY_BAD_EXPR_ENABLED && __TBB_TASK_PRIORITY
     REMARK( "TestPriorityAssertions\n" );
+    tbb::task_scheduler_init init; // to avoid autoinit that'd affect subsequent tests
     tbb::priority_t bad_low_priority = tbb::priority_t( tbb::priority_low - 1 ),
                     bad_high_priority = tbb::priority_t( tbb::priority_high + 1 );
     tbb::task_group_context ctx;
@@ -539,20 +547,57 @@ void TestSetPriority() {
             delete g_trees[t][i];
 }
 }//namespace test_propagation
+
+// TODO: consider common helper for empty bodies, e.g. Harness::DummyBody.
+struct NestedParFor {
+    void operator()(int) const {
+        for (volatile int i = 0; i < 1000; ++i);
+    }
+};
+
+struct OuterParFor {
+    void operator()(int) const {
+        tbb::affinity_partitioner ap;
+        tbb::task_group_context ctx;
+        ctx.set_priority(tbb::priority_high);
+        tbb::parallel_for(0, 100, NestedParFor(), ap, ctx);
+    }
+};
+
+// Test priorities with affinity tasks.
+void TestAffinityTasks() {
+    REMARK("Test priorities with affinity tasks\n");
+    tbb::task_scheduler_init init;
+    tbb::affinity_partitioner ap;
+    for (int i = 0; i < 10; ++i)
+        tbb::parallel_for(0, 100, OuterParFor(), ap);
+}
+
+namespace regression {
+// This is a regression test for a bug with task_group_context used from a thread that created its local scheduler but not the implicit arena
+class TestTGContext {
+public:
+    void operator() (int) const {
+        tbb::task_group_context ctx;
+        ctx.cancel_group_execution();   // initializes the local weak scheduler on the thread
+        ctx.set_priority(tbb::priority_high);
+    }
+};
+
+void TestTGContextOnNewThread() {
+    REMARK("Testing a regression for a bug with task_group_context\n");
+    TestTGContext body;
+    NativeParallelFor(1, body);
+}
+}//namespace regression_priorities
 #endif /* __TBB_TASK_PRIORITY */
 
 #if !__TBB_TEST_SKIP_AFFINITY
 #include "harness_concurrency.h"
 #endif
 
-int TestMain () {
-#if !__TBB_TEST_SKIP_AFFINITY
-    Harness::LimitNumberOfThreads( 16 );
-#endif
-#if !__TBB_TASK_PRIORITY
-    REMARK( "Priorities disabled: Running as just yet another task scheduler test\n" );
-#else
-    test_propagation::TestSetPriority(); // TODO: move down when bug 1996 is fixed
+int RunTests () {
+#if __TBB_TASK_PRIORITY
     TestEnqueueOrder();
 #endif /* __TBB_TASK_PRIORITY */
     TestPriorityAssertions();
@@ -562,6 +607,8 @@ int TestMain () {
     REMARK( "The number of threads: %d\n", P );
     if ( P < 3 )
         return Harness::Skipped;
+    Low = tbb::priority_normal;
+    High = tbb::priority_high;
     TestPeriodicConcurrentActivities();
     TestPrioritySwitchBetweenTwoMasters();
     Low = tbb::priority_low;
@@ -574,9 +621,32 @@ int TestMain () {
     TestPrioritySwitchBetweenTwoMasters();
     PreemptionActivatorId = 1;
     TestPrioritySwitchBetweenTwoMasters();
+    TestAffinityTasks();
+    regression::TestTGContextOnNewThread();
+
     return Harness::Done;
 }
 
+#define TBB_PREVIEW_GLOBAL_CONTROL 1
+#include "tbb/global_control.h"
+
+int TestMain () {
+#if !__TBB_TEST_SKIP_AFFINITY
+    Harness::LimitNumberOfThreads( DesiredNumThreads );
+#endif
+#if !__TBB_TASK_PRIORITY
+    REMARK( "Priorities disabled: Running as just yet another task scheduler test\n" );
+#else
+    test_propagation::TestSetPriority(); // TODO: move down when bug 1996 is fixed
+#endif /* __TBB_TASK_PRIORITY */
+
+    RunTests();
+    tbb::global_control c(tbb::global_control::max_allowed_parallelism, 1);
+    PeriodicActivitiesBody::mode = 1;
+    TestSwitchBetweenMastersRepeats = 1;
+    return RunTests();
+}
+
 #else /* !__TBB_TASK_GROUP_CONTEXT */
 
 int TestMain () {
