@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2016 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_graph.h"
@@ -31,7 +23,7 @@
 #include "tbb/task_scheduler_init.h"
 
 #define N 300
-#define T 4 
+#define T 4
 #define M 4
 
 template< typename R >
@@ -56,7 +48,7 @@ void simple_read_write_tests() {
         for (int i = 0; i < M; ++i) {
            tbb::flow::make_edge( n, r[i] );
         }
-      
+
         if ( t%2 ) {
             for (int i = 0; i < M; ++i) {
                  size_t c = r[i].my_count;
@@ -68,7 +60,7 @@ void simple_read_write_tests() {
             R v1(static_cast<R>(i));
 
             bool result = n.try_put( v1 );
-            if ( !(t%2) && i == 1 ) 
+            if ( !(t%2) && i == 1 )
                 ASSERT( result == true, NULL );
             else
                 ASSERT( result == false, NULL );
@@ -78,9 +70,9 @@ void simple_read_write_tests() {
             for (int j = 0; j < N; ++j ) {
                 R v2(0);
                 ASSERT( n.try_get( v2 ), NULL );
-                if ( t%2 ) 
+                if ( t%2 )
                     ASSERT( R(N+1) == v2, NULL );
-                else 
+                else
                     ASSERT( R(1) == v2, NULL );
             }
         }
@@ -160,7 +152,7 @@ void parallel_read_write_tests() {
     }
 }
 
-int TestMain() { 
+int TestMain() {
     simple_read_write_tests<int>();
     simple_read_write_tests<float>();
     for( int p=MinThread; p<=MaxThread; ++p ) {
@@ -168,6 +160,9 @@ int TestMain() {
         parallel_read_write_tests<int>();
         parallel_read_write_tests<float>();
     }
+#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
+    test_extract_on_node<tbb::flow::write_once_node, int>();
+#endif
     return Harness::Done;
 }
 
